package com.bjlt.spider.commonhandler;

import com.bjlt.spider.common.PlatformCids;
import com.bjlt.spider.common.RedisKeyNames;
import com.bjlt.spider.entity.*;
import com.bjlt.spider.policy.service.PolicyService;
import com.bjlt.spider.policy.service.RefundRulesService;
import com.bjlt.spider.qunaer.entity.QunaerRule;
import com.bjlt.spider.qunaer.entity.QunarRoutings;
import com.bjlt.spider.util.UUIDUtils;
import com.bjlt.spider.utils.CreateDataUtils;
import com.bjlt.spider.utils.CurrencyRateUtils;
import com.bjlt.spider.utils.PolicyCacheKeyUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static java.util.stream.Collectors.toList;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 15:24 2017/10/26
 */
@Slf4j
public abstract class AbstractHandler {


    @Autowired
    private CurrencyRateUtils currencyRateUtils;

    @Autowired
    private PolicyService policyService;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private RefundRulesService refundRulesService;

    @Getter
    @Setter
    protected AbstractHandler nextHandler;


    // private static DateFormat df = ;


    /**
     * 抽象方法 处理请求
     *
     * @param
     * @return
     */
    public abstract List<QunarRoutings> handlerList(List<SearchResponseEntity> list, String cid, QunarRoutings routings, String tripType);

    /**
     * 抽象方法 处理请求
     *
     * @param
     * @return
     */
    public List<QunarRoutings> handlerRequest(List<SearchResponseEntity> list, String cid, QunarRoutings routings, String tripType) {


        for (SearchResponseEntity responseEntity : list) {
            if (responseEntity.getRetSegments().size() == 0) {
                for (AirlineInfomation airlineInfomation : responseEntity.getFromSegments()) {
                    List<PolicyEntity> policyS = findPolicyS(airlineInfomation, cid);
                    Map<String, List<PolicyEntity>> policyEntityListMap = fillterPolicy(policyS, airlineInfomation);
                    policyPrice(airlineInfomation, policyEntityListMap, cid);
                }


            } else {
                for (AirlineInfomation airlineInfomation : responseEntity.getFromSegments()) {
                    List<PolicyEntity> policyS = findPolicyS(airlineInfomation, cid);
                    Map<String, List<PolicyEntity>> policyEntityListMap = fillterPolicy(policyS, airlineInfomation);
                    policyPrice(airlineInfomation, policyEntityListMap, cid);
                }

                for (AirlineInfomation airlineInfomation : responseEntity.getRetSegments()) {
                    List<PolicyEntity> policyS = findPolicyS(airlineInfomation, cid);
                    Map<String, List<PolicyEntity>> policyEntityListMap = fillterPolicy(policyS, airlineInfomation);
                    policyPrice(airlineInfomation, policyEntityListMap, cid);
                }

            }
        }


        List<QunarRoutings> qunarRoutings = descartesLines(list, tripType);

        for (QunarRoutings qunarRouting : qunarRoutings) {
            List<RouteInfoEntity> fromSegments = qunarRouting.getFromSegments();
            if (fromSegments != null) {
                for (RouteInfoEntity routeInfoEntity : fromSegments) {
                    if (cid.equals(PlatformCids.QUNA)) {
                        if ("Y".equals(routeInfoEntity.getCabinClass())) {
                            routeInfoEntity.setCabinClass("1");
                        } else if ("C".equals(routeInfoEntity.getCabinClass())) {
                            routeInfoEntity.setCabinClass("2");
                        } else if ("F".equals(routeInfoEntity.getCabinClass())) {
                            routeInfoEntity.setCabinClass("3");
                        }
                    }
                }
            }


            List<RouteInfoEntity> retSegments = qunarRouting.getRetSegments();
            if (retSegments != null && retSegments.size() != 0) {
                for (RouteInfoEntity retSegment : retSegments) {
                    if (cid.equals(PlatformCids.QUNA)) {
                        if ("Y".equals(retSegment.getCabinClass())) {
                            retSegment.setCabinClass("1");
                        } else if ("C".equals(retSegment.getCabinClass())) {
                            retSegment.setCabinClass("2");
                        } else if ("F".equals(retSegment.getCabinClass())) {
                            retSegment.setCabinClass("3");
                        }
                    }
                }
            }


        }
        return qunarRoutings;

    }


    /**
     * 找到·所有的政策
     *
     * @param infomation
     * @param cid
     * @return
     */
    protected List<PolicyEntity> findPolicyS(AirlineInfomation infomation, String cid) {
        String key = RedisKeyNames.POLICY + PolicyCacheKeyUtil.getkey(infomation, cid);

        List<PolicyEntity> policyEntityList = (List<PolicyEntity>) redisTemplate.opsForValue().get(key);


        if (policyEntityList == null || policyEntityList.size() == 0) {

            policyEntityList = policyService.listPolicyEntityWithCondition(infomation, cid);

            if (policyEntityList.size() != 0) {
                redisTemplate.opsForValue().set(key, policyEntityList);
            }
        }
        return policyEntityList;
    }

    /**
     * 过滤政策 第一步高低价
     *
     * @param policys
     * @param infomation
     * @return
     */
    protected Map<String, List<PolicyEntity>> fillterPolicy(List<PolicyEntity> policys, AirlineInfomation infomation) {

        Map<String, List<PolicyEntity>> retMap = Maps.newHashMap();

        List<PolicyEntity> ECCollect = Lists.newArrayList();

        List<PolicyEntity> EPCollect = Lists.newArrayList();

        List<PolicyEntity> HFCollect = Lists.newArrayList();


        List<PolicyEntity> PMCollect = Lists.newArrayList();


        //首先看有多少种报价信息  最多四种

        List<AirlinePriceInfo> airlinePriceInfos = infomation.getAirlinePriceInfos();


        for (AirlinePriceInfo airlinePriceInfo : airlinePriceInfos) {

            if ("0".equals(airlinePriceInfo.getCabinclass())) {
                Stream<PolicyEntity> ec = policys.stream().filter(a -> "0".equals(a.getCabinScope()) || "9".equals(a.getCabinScope()));

                ECCollect = getPolicyByStream(ec, airlinePriceInfo, infomation);

                retMap.put("0", commonFilter(ECCollect, infomation));
            } else if ("1".equals(airlinePriceInfo.getCabinclass())) {
                Stream<PolicyEntity> ec = policys.stream().filter(a -> "1".equals(a.getCabinScope()) || "9".equals(a.getCabinScope()));

                EPCollect = getPolicyByStream(ec, airlinePriceInfo, infomation);

                retMap.put("1", commonFilter(EPCollect, infomation));
            } else if ("2".equals(airlinePriceInfo.getCabinclass())) {
                Stream<PolicyEntity> ec = policys.stream().filter(a -> "2".equals(a.getCabinScope()) || "9".equals(a.getCabinScope()));

                HFCollect = getPolicyByStream(ec, airlinePriceInfo, infomation);

                retMap.put("2", commonFilter(HFCollect, infomation));
            } else if ("3".equals(airlinePriceInfo.getCabinclass())) {
                Stream<PolicyEntity> ec = policys.stream().filter(a -> "3".equals(a.getCabinScope()) || "9".equals(a.getCabinScope()));

                PMCollect = getPolicyByStream(ec, airlinePriceInfo, infomation);
                retMap.put("3", commonFilter(PMCollect, infomation));

            }


        }
        return retMap;
    }


    public List<PolicyEntity> getPolicyByStream(Stream<PolicyEntity> stream, AirlinePriceInfo airlinePriceInfo, AirlineInfomation infomation) {
        return stream.filter(a ->

                a.getUpPrice().compareTo(new BigDecimal(0)) == 0 || (currencyRateUtils.changeToRmb(infomation.getCurrency(), airlinePriceInfo.getAdultPrice().add(airlinePriceInfo.getAdultTax()), infomation.getAirSourceName()).compareTo(a.getUpPrice()) < 0)


        ).filter(b ->
                b.getLowPrice().compareTo(new BigDecimal(0)) == 0 || (currencyRateUtils.changeToRmb(infomation.getCurrency(), airlinePriceInfo.getAdultPrice().add(airlinePriceInfo.getAdultTax()), infomation.getAirSourceName()).compareTo(b.getLowPrice()) > 0)

        ).filter(c -> {

                    if (c.getExcludeFromDateStart() != null && c.getExcludeFromDateEnd() != null) {
                        Date start = null;
                        Date end = null;
                        Date fromDate = null;
                        try {
                            start = new SimpleDateFormat("yyyy-MM-dd").parse(c.getExcludeFromDateStart());
                            end = new SimpleDateFormat("yyyy-MM-dd").parse(c.getExcludeFromDateEnd());

                            fromDate = new SimpleDateFormat("yyyy-MM-dd").parse(infomation.getFromDate());
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                        return !belongCalendar(fromDate, start, end);
                    } else {
                        return true;
                    }
                }
        ).collect(toList());

    }

    public static boolean belongCalendar(Date time, Date from, Date to) {
        Calendar date = Calendar.getInstance();
        date.setTime(time);

        Calendar after = Calendar.getInstance();
        after.setTime(from);

        Calendar before = Calendar.getInstance();
        before.setTime(to);

        if (date.after(after) && date.before(before)) {
            return true;
        } else if (date.compareTo(after) == 0 || date.compareTo(before) == 0) {
            return true;
        } else {
            return false;
        }
    }


    public List<PolicyEntity> commonFilter(List<PolicyEntity> policyEntities, AirlineInfomation infomation) {
        return policyEntities.stream().filter(p -> {
            if (p.getFlightNumberLimitType().equals(0)) {
                return true;
            }
            if (p.getFlightNumberLimitType().equals(1)) {
                //判断是否中转
                if (infomation.getRouteInfoEntities().size() == 1) {
                    if (Arrays.asList(p.getFlightNumberLimit().split(",")).contains(infomation.getRouteInfoEntities().get(0).getFlightNumber())) {
                        return true;
                    }

                }
                if (infomation.getRouteInfoEntities().size() > 1) {
                    StringBuffer sb = new StringBuffer();
                    infomation.getRouteInfoEntities().stream().forEach(k -> sb.append(k.getFlightNumber() + ","));
                    String string = sb.toString();
                    string = string.substring(0, string.length() - 1);
                    List collectList = Arrays.stream(p.getFlightNumberLimit().split("&")).collect(toList());
                    return collectList.contains(string);
                }
            }
            return false;
        }).collect(Collectors.toList());
    }


    /**
     * 按照·政策调整价格
     *
     * @param infomation
     * @param
     * @param cid
     * @return
     */
    protected AirlineInfomation policyPrice(AirlineInfomation
                                                    infomation, Map<String, List<PolicyEntity>> policyEntityListMap, String cid) {


        List<PolicyEntity> ECPolicyEntityList = policyEntityListMap.get("0");

        List<PolicyEntity> EPPolicyEntityList = policyEntityListMap.get("1");

        List<PolicyEntity> HFPolicyEntityList = policyEntityListMap.get("2");

        List<PolicyEntity> PMPolicyEntityList = policyEntityListMap.get("3");


        String groupId = infomation.getAirSourceName();

        List<AirlinePriceInfo> airlinePriceInfos = infomation.getAirlinePriceInfos();

        List<AirlinePriceInfo> afterPolicy = Lists.newArrayList();

        String fromDate = infomation.getFromDate();

        for (AirlinePriceInfo airlinePriceInfo : airlinePriceInfos) {

            if (airlinePriceInfo.getCabinclass().equals("0")) {
                AirlinePriceInfo ECAirlinePriceInfo = policyPrice(airlinePriceInfo, ECPolicyEntityList, cid, groupId, fromDate, infomation.getCurrency());

                afterPolicy.add(ECAirlinePriceInfo);
            } else if (airlinePriceInfo.getCabinclass().equals("1")) {
                AirlinePriceInfo EPAirlinePriceInfo = policyPrice(airlinePriceInfo, EPPolicyEntityList, cid, groupId, fromDate, infomation.getCurrency());

                afterPolicy.add(EPAirlinePriceInfo);
            } else if (airlinePriceInfo.getCabinclass().equals("2")) {
                AirlinePriceInfo HFAirlinePriceInfo = policyPrice(airlinePriceInfo, HFPolicyEntityList, cid, groupId, fromDate, infomation.getCurrency());

                afterPolicy.add(HFAirlinePriceInfo);
            } else if (airlinePriceInfo.getCabinclass().equals("3")) {
                AirlinePriceInfo PMAirlinePriceInfo = policyPrice(airlinePriceInfo, PMPolicyEntityList, cid, groupId, fromDate, infomation.getCurrency());
                afterPolicy.add(PMAirlinePriceInfo);
            }
        }

        infomation.setAirlinePriceInfos(afterPolicy);


        return infomation;


    }

    /**
     * 调价
     *
     * @param priceInfo
     * @param policyListAfterFilter
     * @param cid
     * @param groupId
     * @param fromDate
     * @param currency
     * @param
     * @return
     */
    protected AirlinePriceInfo policyPrice(AirlinePriceInfo priceInfo, List<PolicyEntity> policyListAfterFilter, String cid, String groupId, String fromDate, String
            currency) {


        //开始转换

        BigDecimal adultPrice1 = priceInfo.getAdultPrice().setScale(0, BigDecimal.ROUND_HALF_UP);

        BigDecimal adultTax1 = priceInfo.getAdultTax().setScale(0, BigDecimal.ROUND_HALF_UP);


        priceInfo.setAdultPrice(currencyRateUtils.changeToRmb(currency, adultPrice1, groupId));

        priceInfo.setAdultTax(currencyRateUtils.changeToRmb(currency, adultTax1, groupId));

        BigDecimal childPrice1 = priceInfo.getChildPrice().setScale(0, BigDecimal.ROUND_HALF_UP);
        if (childPrice1 != null && new BigDecimal(0).compareTo(childPrice1) != 0) {
            priceInfo.setChildPrice(currencyRateUtils.changeToRmb(currency, childPrice1, groupId));
            BigDecimal childTax = priceInfo.getChildTax().setScale(0, BigDecimal.ROUND_HALF_UP);
            priceInfo.setChildTax(currencyRateUtils.changeToRmb(currency, childTax, groupId));
        } else {
            priceInfo.setChildPrice(priceInfo.getAdultPrice());
            priceInfo.setChildTax(priceInfo.getAdultTax());
        }


        Map<BigDecimal, String> priceMap = new HashMap<>();

        if (policyListAfterFilter.size() != 0) {
            for (PolicyEntity policyEntity : policyListAfterFilter) {
                BigDecimal colPrice = new BigDecimal(0);

                BigDecimal finalTax = new BigDecimal(0);
                //拿到原始价格
                BigDecimal adultPrice = priceInfo.getAdultPrice();

                BigDecimal adultTax = priceInfo.getAdultTax();


                // 0  是 固定价格  成人价格是 固定价格的百分之70  儿童也一样
                if (policyEntity.getAdjustType().equals("0")) {
                    BigDecimal finalPrice = policyEntity.getFinalPrice();
                    colPrice = finalPrice.multiply(new BigDecimal(0.7));

                    finalTax = finalPrice.multiply(new BigDecimal(0.3));

                } else {
                    //1 是调价百分比 然后加减钱
                    BigDecimal radio = new BigDecimal(policyEntity.getAdjustRadio().trim());

                    BigDecimal divide = new BigDecimal(100).add(radio).divide(new BigDecimal(100));


                    colPrice = adultPrice.multiply(divide);


                    finalTax = adultTax.multiply(divide);

                }
                //然后调
                if (!StringUtils.isBlank(policyEntity.getAddOrSubtractMoney().trim())) {
                    colPrice = colPrice.add(new BigDecimal(policyEntity.getAddOrSubtractMoney()));
                }

                priceMap.put(colPrice, policyEntity.getId() + "&" + finalTax);
            }
            //然后拿出最小的  最有优势的价格
            BigDecimal bigDecimal = priceMap.keySet().stream().sorted().collect(toList()).get(0);


            String idAndTax = priceMap.get(bigDecimal);

            String id = idAndTax.split("&")[0];

            BigDecimal tax = new BigDecimal(idAndTax.split("&")[1]);

            priceInfo.setAdultPrice(bigDecimal.setScale(0, BigDecimal.ROUND_HALF_UP));

            priceInfo.setChildPrice(bigDecimal.setScale(0, BigDecimal.ROUND_HALF_UP));

            priceInfo.setAdultTax(tax.setScale(0, BigDecimal.ROUND_HALF_UP));

            priceInfo.setChildTax(tax.setScale(0, BigDecimal.ROUND_HALF_UP));

            priceInfo.setPolicyId(Integer.valueOf(id));

        }


//        //最后做全局调价
        List<ScGlobalRegulate> regulates = (List<ScGlobalRegulate>) redisTemplate.opsForValue().get("REGULATEMONEY-" + cid + groupId);

        if (regulates != null) {
            regulates = regulates.stream().filter(t -> t.getStatus().equals(1)).filter(s -> s.getProductType().equals(priceInfo.getCabinclass()) || s.getProductType().equals("9")).collect(toList());
        }
        if (regulates != null) {
            Map<BigDecimal, BigDecimal> globalPriceMap = new HashMap<>();
            for (ScGlobalRegulate regulate : regulates) {

                Date start = null;
                Date end = null;
                Date flyDate = null;

                try {


                    start = new SimpleDateFormat("yyyy-MM-dd").parse(regulate.getTimeStart());

                    end = new SimpleDateFormat("yyyy-MM-dd").parse(regulate.getTimeEnd());

                    flyDate = new SimpleDateFormat("yyyy-MM-dd").parse(fromDate);


                    if (belongCalendar(flyDate, start, end)) {

                        BigDecimal adultPrice = priceInfo.getAdultPrice().add(new BigDecimal(regulate.getAdultPrice()));
                        priceInfo.setAdultPrice(adultPrice);


                        if (regulate.getChildPrice() != null && priceInfo.getChildPrice().compareTo(new BigDecimal(0)) != 0) {
                            priceInfo.setChildPrice(priceInfo.getChildPrice().add(new BigDecimal(regulate.getChildPrice())));
                        }

                        globalPriceMap.put(priceInfo.getAdultPrice(), priceInfo.getChildPrice());

                    }


                } catch (Exception e) {
                    log.error("错误对象{}");
                    e.printStackTrace();
                }
            }
            if (globalPriceMap.size() != 0) {
                BigDecimal adultPrice = globalPriceMap.keySet().stream().sorted().collect(toList()).get(0);
                BigDecimal childPrice = globalPriceMap.get(adultPrice);
                priceInfo.setAdultPrice(adultPrice);

                priceInfo.setChildPrice(childPrice);
            }


        }


        return priceInfo;
    }

    /**
     * 笛卡尔积
     *
     * @param list
     * @return
     */
    protected List<QunarRoutings> descartesLines(List<SearchResponseEntity> list, String tripType) {
        //迭代 将所有的去程拿出来
        List<AirlineInfomation> fromList = Lists.newArrayList();
        //将所有的返程拿出来
        List<AirlineInfomation> retList = Lists.newArrayList();


        list.stream().forEach(t ->
                fromList.addAll(t.getFromSegments())

        );

        if (list.get(0).getRetSegments().size() != 0) {
            list.stream().forEach(t ->
                    retList.addAll(t.getRetSegments())

            );
        }
        List<QunarRoutings> qunarRoutingsList = Lists.newArrayList();
        //只有单程 没有往返 不会存在有笛卡尔积的可能
        //log.info("此次航班类型是{}", tripType);
        if ("1".equals(tripType) && fromList.size() != 0) {


            for (AirlineInfomation airlineInfomation : fromList) {


                String uuid = UUIDUtils.getUUID();
                //查看有几种报价
                List<AirlinePriceInfo> airlinePriceInfos = airlineInfomation.getAirlinePriceInfos();
                for (AirlinePriceInfo airlinePriceInfo : airlinePriceInfos) {


                    QunarRoutings qunarRoutings = new QunarRoutings();

                    List<RouteInfoEntity> routeInfoEntities = airlineInfomation.getRouteInfoEntities();


                    List<RouteInfoEntity> newRouteList = new ArrayList<>();

                    for (RouteInfoEntity routeInfoEntity : routeInfoEntities) {
                        try {
                            newRouteList.add(routeInfoEntity.clone());
                        } catch (CloneNotSupportedException e) {
                            e.printStackTrace();
                        }
                    }


                    for (RouteInfoEntity routeInfoEntity : newRouteList) {
                        if (routeInfoEntity.getStopCities() == null) {
                            routeInfoEntity.setStopCities("");
                        }
                    }


                    if ("0".equals(airlinePriceInfo.getCabinclass())) {

                        qunarRoutings.setTag(0);
                        newRouteList.forEach(t -> {
                            t.setCabinClass("Y");
                            t.setCabin("E");
                        });
                    } else if ("1".equals(airlinePriceInfo.getCabinclass())) {
                        qunarRoutings.setTag(1);
                        newRouteList.forEach(t -> {
                            t.setCabinClass("Y");
                            t.setCabin("V");
                        });
                    } else if ("2".equals(airlinePriceInfo.getCabinclass())) {
                        qunarRoutings.setTag(2);
                        newRouteList.forEach(t -> {
                            t.setCabinClass("C");
                            t.setCabin("Y");
                        });
                    } else if ("3".equals(airlinePriceInfo.getCabinclass())) {
                        qunarRoutings.setTag(3);
                        newRouteList.forEach(t -> {
                            t.setCabinClass("F");
                            t.setCabin("F");
                        });
                    }


                    qunarRoutings.setUuid(uuid);

                    qunarRoutings.setPolicyID(String.valueOf(airlinePriceInfo.getPolicyId()));

                    qunarRoutings.setData(CreateDataUtils.getData(airlineInfomation, airlinePriceInfo, null, String.valueOf(airlinePriceInfo.getPolicyId()), qunarRoutings.getTag(), null, airlinePriceInfo.getMaxSeats()));

                    qunarRoutings.setCurreny(airlineInfomation.getCurrency());

                    qunarRoutings.setAdultPrice(airlinePriceInfo.getAdultPrice());

                    qunarRoutings.setAdultTax(airlinePriceInfo.getAdultTax());

                    qunarRoutings.setChildPrice(airlinePriceInfo.getChildPrice());

                    qunarRoutings.setChildTax(airlinePriceInfo.getChildTax());

                    qunarRoutings.setMaxSeats(airlinePriceInfo.getMaxSeats());

                    qunarRoutings.setTripType("1");


                    qunarRoutings.setFromCity(airlineInfomation.getFormCity());


                    qunarRoutings.setToCity(airlineInfomation.getToCity());

                    qunarRoutings.setFromDate(airlineInfomation.getFromDate());

                    qunarRoutings.setRetDate(airlineInfomation.getRetDate());

                    commonInsert(qunarRoutings);

                    RuleEntity ruleEntity = insertRefundInfo(airlineInfomation.getAirSourceName(), qunarRoutings.getTag());
                    qunarRoutings.setRule(ruleEntity);


                    qunarRoutings.setFromSegments(newRouteList);


                    List<RouteInfoEntity> RouteInfoEntitys = new ArrayList<>();
                    qunarRoutings.setRetSegments(RouteInfoEntitys);


                    qunarRoutingsList.add(qunarRoutings);

                }

            }
            return qunarRoutingsList;

        } else {
            log.info("此次航班类型是{}", "往返");

            for (AirlineInfomation fromInfomation : fromList) {

                String fromUuid = UUIDUtils.getUUID();

                List<AirlinePriceInfo> fromPriceInfos = fromInfomation.getAirlinePriceInfos();

                String fromAirSourceName = fromInfomation.getAirSourceName();

                for (AirlinePriceInfo fromPriceInfo : fromPriceInfos) {

                    String retUuid = UUIDUtils.getUUID();


                    for (AirlineInfomation retInfomation : retList) {

                        String retAirSourceName = retInfomation.getAirSourceName();


                        List<AirlinePriceInfo> retPriceInfos = retInfomation.getAirlinePriceInfos();


                        for (AirlinePriceInfo retPriceInfo : retPriceInfos) {


                            List<RouteInfoEntity> fromRouteInfoEntities = fromInfomation.getRouteInfoEntities();


                            List<RouteInfoEntity> retRouteInfoEntities = retInfomation.getRouteInfoEntities();


                            List<RouteInfoEntity> fromNewRouteList = new ArrayList<>();

                            List<RouteInfoEntity> retNewRouteList = new ArrayList<>();

                            for (RouteInfoEntity routeInfoEntity : fromRouteInfoEntities) {
                                try {
                                    fromNewRouteList.add(routeInfoEntity.clone());
                                } catch (CloneNotSupportedException e) {
                                    e.printStackTrace();
                                }
                            }

                            for (RouteInfoEntity routeInfoEntity : retRouteInfoEntities) {
                                try {
                                    retNewRouteList.add(routeInfoEntity.clone());
                                } catch (CloneNotSupportedException e) {
                                    e.printStackTrace();
                                }
                            }

                            Integer fromPriceClass = Integer.valueOf(fromPriceInfo.getCabinclass());

                            Integer retPriceClass = Integer.valueOf(retPriceInfo.getCabinclass());


                            if (fromAirSourceName.equals(retAirSourceName) && fromPriceClass.equals(0) && retPriceClass.equals(0)) {

                                if ((fromPriceClass <= retPriceClass) || (fromNewRouteList.size() == 0 && retNewRouteList.size() != 0)
                                        || (fromNewRouteList.size() != 0 && retNewRouteList.size() == 0)) {


                                    QunarRoutings qunarRoutings = new QunarRoutings();


                                    for (RouteInfoEntity routeInfoEntity : fromNewRouteList) {
                                        if (routeInfoEntity.getStopCities() == null) {
                                            routeInfoEntity.setStopCities("");
                                        }
                                    }

                                    for (RouteInfoEntity routeInfoEntity : retNewRouteList) {
                                        if (routeInfoEntity.getStopCities() == null) {
                                            routeInfoEntity.setStopCities("");
                                        }
                                    }

                                    if ("0".equals(fromPriceInfo.getCabinclass())) {
                                        qunarRoutings.setTag(0);
                                        fromNewRouteList.stream().forEach(t -> {
                                            t.setCabinClass("Y");
                                            t.setCabin("E");
                                        });
                                        retNewRouteList.stream().forEach(t -> {
                                            t.setCabinClass("Y");
                                            t.setCabin("E");
                                        });
                                    } else if ("1".equals(fromPriceInfo.getCabinclass())) {
                                        qunarRoutings.setTag(1);
                                        fromNewRouteList.stream().forEach(t -> {
                                            t.setCabinClass("Y");
                                            t.setCabin("V");
                                        });
                                        retNewRouteList.stream().forEach(t -> {
                                            t.setCabinClass("Y");
                                            t.setCabin("V");
                                        });
                                    } else if ("2".equals(fromPriceInfo.getCabinclass())) {
                                        qunarRoutings.setTag(2);
                                        fromNewRouteList.stream().forEach(t -> {
                                            t.setCabinClass("C");
                                            t.setCabin("Y");
                                        });
                                        retNewRouteList.stream().forEach(t -> {
                                            t.setCabinClass("C");
                                            t.setCabin("Y");
                                        });
                                    } else if ("3".equals(fromPriceInfo.getCabinclass())) {
                                        qunarRoutings.setTag(3);
                                        fromNewRouteList.stream().forEach(t -> {
                                            t.setCabinClass("F");
                                            t.setCabin("F");
                                        });
                                        retNewRouteList.stream().forEach(t -> {
                                            t.setCabinClass("F");
                                            t.setCabin("F");
                                        });
                                    }
                                    qunarRoutings.setRetTag(Integer.valueOf(retPriceInfo.getCabinclass()));
                                    qunarRoutings.setPolicyID(String.valueOf(fromPriceInfo.getPolicyId()) + "&" + String.valueOf(retPriceInfo.getPolicyId()));


                                    qunarRoutings.setCurreny(fromInfomation.getCurrency());

                                    qunarRoutings.setAdultPrice(fromPriceInfo.getAdultPrice().add(retPriceInfo.getAdultPrice()));

                                    qunarRoutings.setAdultTax(fromPriceInfo.getAdultTax().add(retPriceInfo.getAdultTax()));

                                    qunarRoutings.setChildPrice(fromPriceInfo.getChildPrice().add(retPriceInfo.getChildPrice()));

                                    qunarRoutings.setChildTax(fromPriceInfo.getChildTax().add(retPriceInfo.getChildTax()));

                                    qunarRoutings.setUuid(fromUuid + retUuid);

                                    qunarRoutings.setFromCity(fromInfomation.getFormCity());

                                    qunarRoutings.setToCity(fromInfomation.getToCity());

                                    qunarRoutings.setFromDate(fromInfomation.getFromDate());

                                    qunarRoutings.setRetDate(fromInfomation.getRetDate());

                                    qunarRoutings.setMaxSeats(fromPriceInfo.getMaxSeats() <= retPriceInfo.getMaxSeats() ? fromPriceInfo.getMaxSeats() : retPriceInfo.getMaxSeats());

                                    qunarRoutings.setData(CreateDataUtils.getData(fromInfomation, fromPriceInfo, retPriceInfo, fromPriceInfo.getPolicyId() + "&" + retPriceInfo.getPolicyId(), qunarRoutings.getTag(), Integer.valueOf(retPriceInfo.getCabinclass()), qunarRoutings.getMaxSeats()));


                                    commonInsert(qunarRoutings);
                                    qunarRoutings.setRule(insertRefundInfo(fromInfomation.getAirSourceName(), qunarRoutings.getTag()));

                                    qunarRoutings.setTripType("2");

                                    qunarRoutings.setFromSegments(fromNewRouteList);
                                    qunarRoutings.setRetSegments(retNewRouteList);
                                    qunarRoutingsList.add(qunarRoutings);
                                }
                            }
                        }
                    }
                }
            }
            return qunarRoutingsList;
        }

    }


    /**
     * 匹配退改签
     */

    public RuleEntity insertRefundInfo(String airSourceName, Integer tag) {



        String key = RedisKeyNames.REFUND + airSourceName + "OBJ-" +  "-" + tag;

        RuleEntity refundRulesUse = (RuleEntity) redisTemplate.opsForValue().get(key);


        if (refundRulesUse == null) {
            List<QunaerRule> rules = refundRulesService.listRefundRules(QunaerRule.builder().sourceGroupId(Integer.valueOf(airSourceName)).tag(tag).build());

            QunaerRule qunaerRule = rules.get(0);

            refundRulesUse = RuleEntity.builder()
                    .baggage(qunaerRule.getBaggage())
                    .endorse(qunaerRule.getEndorse())
                    .endorsement(qunaerRule.getEndorsement())
                    .hasBaggage(0)
                    .hasEndorse(qunaerRule.getHasEndorse())
                    .hasNoShow(qunaerRule.getHasNoShow())
                    .noShowLimitTime(qunaerRule.getNoShowLimitTime())
                    .partEndorse(qunaerRule.getPartEndorse())
                    .specialNoShow(qunaerRule.getSpecialNoShow())
                    .hasRefund(qunaerRule.getHasRefund())
                    .other(qunaerRule.getOther())
                    .refund(qunaerRule.getRefund())
                    .partRefundPrice(qunaerRule.getPartEndorsePrice())
                    .penalty(qunaerRule.getPenalty())
                    .partEndorsePrice(qunaerRule.getPartEndorsePrice())
                    .partRefund(qunaerRule.getPartRefund())
                    .build();
            redisTemplate.opsForValue().set(key, refundRulesUse);


        }

        return refundRulesUse;
    }


    private void commonInsert(QunarRoutings qunarRoutings) {
        qunarRoutings.setNationality("");
        qunarRoutings.setNationalityType(0);
        qunarRoutings.setSuitAge("");
        qunarRoutings.setPriceType(0);
        qunarRoutings.setApplyType(0);
        qunarRoutings.setAdultTaxType(0);
        qunarRoutings.setChildTaxType(0);
        qunarRoutings.setTicketTimeLimit(10080);
        qunarRoutings.setTicketInvoiceType(0);
    }

//    public String changeCabin(String cabin){
//        if (cabin.equals("EC")){
//
//        }
//    }

}
