package com.bjlt.spider.commonhandler;

import com.bjlt.spider.entity.SearchResponseEntity;
import com.bjlt.spider.qunaer.entity.PriceMap;
import com.bjlt.spider.qunaer.entity.QunarRoutings;
import com.bjlt.spider.qunaer.entity.VerifyRedisLog;
import com.bjlt.spider.util.UUIDUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 15:51 2017/10/26
 */
@Component
@Slf4j
public class AirSoucePolicyForVerifyHandler extends AbstractHandler {

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;


    private ObjectMapper objectMapper = new ObjectMapper();

    @Override
    public List<QunarRoutings> handlerList(List<SearchResponseEntity> list, String cid, QunarRoutings qunarRoutings, String tripType) {
        List<QunarRoutings> QunarRoutingsInSearch = handlerRequest(list, cid, qunarRoutings, tripType);

        String fromAirCode = qunarRoutings.getFromSegments().get(0).getAircraftCode();


        String retAirCode = "";

        if (qunarRoutings.getRetSegments() != null && qunarRoutings.getRetSegments().size() != 0) {
            retAirCode = qunarRoutings.getRetSegments().get(0).getAircraftCode();
        }

        qunarRoutings.setUuid(null);

        qunarRoutings.setData(null);

        qunarRoutings.setFromDate(qunarRoutings.getFromDate());

        if (qunarRoutings.getFromSegments() != null) {
            qunarRoutings.getFromSegments().stream().forEach(t -> {
                t.setAircraftCode("");
                t.setCabinClass(null);
                t.setStopCitiesEntity(null);
                t.setStopCities("");
                t.setDuration(null);


            });

        }


        if (qunarRoutings.getRetDate() != null) {

            if (!"".equals(qunarRoutings.getRetDate())) {
                qunarRoutings.setRetDate(qunarRoutings.getRetDate());
                qunarRoutings.getRetSegments().stream().forEach(t -> {
                    t.setCabinClass(null);
                    t.setAircraftCode("");
                    t.setDuration(null);
                    t.setStopCitiesEntity(null);
                    t.setStopCities("");
                });
            }

        }


        QunarRoutingsInSearch.stream().forEach(t -> {
            t.setUuid(null);

            t.setData(null);


            if (t.getFromSegments() != null) {
                t.getFromSegments().stream().forEach(s -> {
                    s.setStopCities("");
                    s.setCabinClass(null);
                    s.setAircraftCode("");
                    s.setDuration(null);
                    s.setStopCitiesEntity(null);
                });
            }


            if (t.getRetSegments() != null) {
                t.getRetSegments().stream().forEach(s -> {
                    s.setAircraftCode("");
                    s.setCabinClass(null);
                    s.setDuration(null);
                    s.setStopCitiesEntity(null);
                    s.setStopCities("");
                });
            }
        });


        List<QunarRoutings> collect = Lists.newArrayList();
        if (qunarRoutings.getTripType().equals("1")) {
            collect = QunarRoutingsInSearch.stream().filter(t ->

                    t.getFromDate().equals(qunarRoutings.getFromDate())
            ).filter(k -> k.getFromSegments().containsAll(qunarRoutings.getFromSegments()))
                    .collect(toList());


        } else {
            collect = QunarRoutingsInSearch.stream().filter(t ->

                    t.getFromDate().equals(qunarRoutings.getFromDate())
                            && t.getRetDate().equals(qunarRoutings.getRetDate())


            ).filter(k -> k.getFromSegments().containsAll(qunarRoutings.getFromSegments())
            ).filter(k -> k.getRetSegments().containsAll(qunarRoutings.getRetSegments())
            ).collect(toList());

        }
        if (collect.size() == 0) {

            log.info("验价失败");
            return null;

        } else {
            PriceMap priceMap = qunarRoutings.getPriceMap();

            Integer tag = priceMap.getFromTag();

            Integer retTag = priceMap.getRetTag();

            List<QunarRoutings> collect1 = null;
            if (tripType.equals("2")) {
                collect1 = collect.stream().filter(t -> t.getTag().equals(tag) && t.getRetTag().equals(retTag)).sorted(Comparator.comparing(QunarRoutings::getAdultPrice)).collect(Collectors.toList());

                if (collect1 == null || collect1.size() == 0) {
                    collect1 = collect.stream().filter(t -> t.getTag() >= tag).sorted(Comparator.comparing(QunarRoutings::getAdultPrice)).collect(Collectors.toList());
                }

            } else {
                collect1 = collect.stream().filter(t -> t.getTag().equals(tag)).sorted(Comparator.comparing(QunarRoutings::getAdultPrice)).collect(Collectors.toList());

            }

            QunarRoutings qunarRoutings2 = null;
            if (collect1 != null || collect1.size() != 0) {
                qunarRoutings2 = collect1.get(0);
            } else {
                log.info("验价失败-----tag");
                return null;
            }
            BigDecimal adultPrice = qunarRoutings2.getAdultPrice();

            log.info("v 过程中拿到的 价格 {}", adultPrice);


            String cabin = qunarRoutings2.getFromSegments().get(0).getCabin();

            log.info("验价过程中拿到的 cabin {}", cabin);


            List<QunarRoutings> finalCollect = Lists.newArrayList();
            BigDecimal price = priceMap.getAdultPrice();
            log.info("S 过程中拿到的 价格 {}", price);


            if (adultPrice.subtract(price).compareTo(new BigDecimal(60)) > 0) {
                log.info("---------两次价格差值大于60  强行失败---------");
                return null;
            }

            if (adultPrice.compareTo(price) <= 0) {

                qunarRoutings2.setAdultPrice(price);
                qunarRoutings2.setChildPrice(priceMap.getChildPrice());
                qunarRoutings2.setChildTax(priceMap.getChildTax());
                qunarRoutings2.setAdultTax(priceMap.getAdultTax());
                qunarRoutings2.setData(qunarRoutings.getData());

            }

            qunarRoutings2.getFromSegments().stream().forEach(t -> t.setAircraftCode(fromAirCode));


            if (qunarRoutings2.getRetSegments() != null && qunarRoutings2.getRetSegments().size() != 0) {
                String finalRetAirCode = retAirCode;
                qunarRoutings2.getRetSegments().stream().forEach(t -> t.setAircraftCode(finalRetAirCode));

            }
            finalCollect.add(qunarRoutings2);

            return finalCollect;
        }
    }


    public static void main(String[] args) {
        BigDecimal a1 = new BigDecimal(100);

        BigDecimal a2 = new BigDecimal(20);

        System.out.println(a1.subtract(a2));

    }
}





