package com.bjlt.spider.hangbanguanjia.service.impl;

import com.bjlt.spider.common.PlatformCids;
import com.bjlt.spider.entity.RouteInfoEntity;
import com.bjlt.spider.hangbanguanjia.entity.HbgjFlightSegment;
import com.bjlt.spider.hangbanguanjia.service.HbgjOrderService;
import com.bjlt.spider.interfaces.ControRpc;
import com.bjlt.spider.qunaer.entity.*;
import com.bjlt.spider.qunaer.service.QunaerService;
import com.bjlt.spider.utils.CreateDataUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 14:32 2018/1/4
 */
@Slf4j
@Service
public class HbgjOrderServiceImpl implements HbgjOrderService {

    @Autowired
    private QunaerService qunaerService;


    @Override
    public HbgjOrderRespEntity bookOrder(HttpServletRequest request) {
        QunaerOrderParams qunaerOrderParams = new QunaerOrderParams();
        HbgjOrderRespEntity hbgjOrderRespEntityRespEntity = new HbgjOrderRespEntity();
        try {

            SAXBuilder saxBuilder = new SAXBuilder();
            Document document = saxBuilder.build(request.getInputStream());
            Element rootElement = document.getRootElement();
            log.info("获取到的根元素" + rootElement);

            //航班管家订单编号
            String hbgjOrderID = rootElement.getChildText("hbgjOrderID");

            //结算总价
            String settlePrice = rootElement.getChildText("settlePrice");
            //订单票面总价
            String totalPrice = rootElement.getChildText("totalPrice");
            //去程|回程，OB-去程，IB-回程
            String tripType = rootElement.getChildText("tripType").trim();
            if (tripType.equals("OW")) {
                qunaerOrderParams.setTripType("1");
            } else if (tripType.equals("RT")) {
                qunaerOrderParams.setTripType("2");
            } else {
                if (qunaerOrderParams.getTripType() == null || "".equals(qunaerOrderParams.getTripType())) {
                    hbgjOrderRespEntityRespEntity.setCode(1);
                    hbgjOrderRespEntityRespEntity.setMsg("行程类型不支持");
                    hbgjOrderRespEntityRespEntity.setPnrCode("000000");
                    return hbgjOrderRespEntityRespEntity;
                }
            }
            List<Element> pnrInfos = rootElement.getChild("pnrInfos").getChildren("pnrInfo");

            Element element = pnrInfos.get(0);


            QunarRoutings routings = new QunarRoutings();

            //pnr编码
            element.getChildText("pnrCode");
            //PNR文本信息
            element.getChildText("pnrTxt");
            //大pnr
            element.getChildText("bigPnr");
            //政策id
            String policyId = element.getChildText("policyId");

            String commissionRate = element.getChildText("commissionRate");
            element.getChildText("money");
            //儿童政策id 有儿童必传
            element.getChildText("chdPolicyId");
            //单张成人票面价格不含基建燃油
            BigDecimal price = BigDecimal.valueOf(Double.valueOf(element.getChildText("price")));
            BigDecimal chPrice = BigDecimal.valueOf(Double.valueOf(element.getChildText("chPrice")));

            List<Element> flightSegments = element.getChild("flightSegments").getChildren("flightSegment");

            //联系人信息
            ContactInfo contactInfo = new ContactInfo();
            Element contactInfo1 = rootElement.getChild("contactInfo");
            String personName = contactInfo1.getChildText("personName");
            String mobilePhone = contactInfo1.getChildText("mobilePhone");
            contactInfo.setMobile(mobilePhone);
            contactInfo.setName(personName);
            qunaerOrderParams.setContact(contactInfo);

            makeOrderParams(flightSegments, qunaerOrderParams, routings, price, tripType);


            List<PassengersInfo> passengersInfos = Lists.newArrayList();
            //旅客信息
            List<Element> children = element.getChild("travelerInfos").getChildren("traveler");
            if (children.size() != 0) {
                children.parallelStream().forEach(k -> {
                    PassengersInfo passengersInfo = new PassengersInfo();
                    //旅客姓名
                    passengersInfo.setName(k.getChildText("personName"));
                    //旅客性别 M F
                    passengersInfo.setGender(k.getChildText("gender"));
                    //订位类型  ADT 和CHD-->转为quna 0 成人 1儿童
                    passengersInfo.setAgeType(k.getChildText("type").equals("ADT") ? 0 : 1);
                    //证件类型   0：身份证
                    passengersInfo.setCardType(k.getChildText("idCardType"));
                    //身份证号码 必传
                    passengersInfo.setCardNum(k.getChildText("idCardNo"));
                    //生日  儿童必传
                    passengersInfo.setBirthday(k.getChildText("birthday"));
                    //国籍  非必传
                    passengersInfo.setNationality(k.getChildText("nationality"));
                    passengersInfos.add(passengersInfo);
                });
            }
            PassengersInfo[] objects = passengersInfos.toArray(new PassengersInfo[]{});
            qunaerOrderParams.setPassengers(objects);
            qunaerOrderParams.setCid(PlatformCids.QUNA);
            String data = CreateDataUtils.getDataInHbgj(qunaerOrderParams.getRouting());
            qunaerOrderParams.getRouting().setData(data);
            //解析后，重新搜索，验价
            QunaerOrderRespEntity order = qunaerService.order(qunaerOrderParams, PlatformCids.HBGJ);

            hbgjOrderRespEntityRespEntity.setCode(0);
            hbgjOrderRespEntityRespEntity.setPnrCode(order.getPnrCode());

            hbgjOrderRespEntityRespEntity.setOrderID(order.getOrderNo());

            hbgjOrderRespEntityRespEntity.setTotalSettlePrice(order.getFee().doubleValue());

            return hbgjOrderRespEntityRespEntity;

        } catch (JDOMException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 下单之前验证价格
     * @param request
     * @return
     */
    @Override
    public HbgjOrderRespEntity verifyBeforePay(HttpServletRequest request) {

        try {
            SAXBuilder saxBuilder = new SAXBuilder();
            Document document = saxBuilder.build(request.getInputStream());
            Element rootElement = document.getRootElement();
            log.info("获取到的根元素" + rootElement);

            String orderID = rootElement.getChildText("orderID");


            String hbgjOrderID = rootElement.getChildText("hbgjOrderID");

            String settlePrice = rootElement.getChildText("settlePrice");


            List<Element> flightSegments = rootElement.getChild("flightSegments").getChildren("flightSegment");

            List<HbgjFlightSegment> flightSegmentList = Lists.newArrayList();


        } catch (JDOMException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }


        return null;
    }


    public QunaerOrderParams makeOrderParams(List<Element> flightSegments, QunaerOrderParams qunaerOrderParams, QunarRoutings routings, BigDecimal price, String tripType) {

        if (flightSegments.size() != 0) {
            //去程的信息
            List<RouteInfoEntity> fromSegments = Lists.newArrayList();
            List<RouteInfoEntity> retSegments = Lists.newArrayList();


            for (Element flightSegment : flightSegments) {
                RouteInfoEntity routeInfoEntity = new RouteInfoEntity();

                String fromCity = flightSegment.getChildText("departCityCode");
                String toCity = flightSegment.getChildText("arriveCityCode");
                String fromDate = flightSegment.getChildText("flydate");

                String arrdate = flightSegment.getChildText("arrdate");

                //航班号
                String flyno = flightSegment.getChildText("flyno");
                //子舱位
                String cabin = flightSegment.getChildText("cabin");
                //票面价
                BigDecimal priceMian = BigDecimal.valueOf(Double.valueOf(flightSegment.getChildText("price")));
                //BigDecimal chdprice = BigDecimal.valueOf(Double.valueOf(flightSegment.getChildText("chdprice")));
                //燃油
                // String rateFee = flightSegment.getChildText("rateFee");
                //机场建设费
                //String airportFee = flightSegment.getChildText("airportFee");
                //燃油
                //String chdRateFee = flightSegment.getChildText("chdRateFee");
                //机场建设费
                //String chdAirportFee = flightSegment.getChildText("chdAirportFee");
                String baseCabin = flightSegment.getChildText("baseCabin");
                String deptime = flightSegment.getChildText("deptime");
                String arrtime = flightSegment.getChildText("arrtime");


                routeInfoEntity.setAdultPrice(priceMian);
                routeInfoEntity.setArrAirport(toCity);
                routeInfoEntity.setArrTime(arrtime);
                //cabin 是E,F 对应是1和3  基础仓位是Y 子舱为就是E  基础仓位是C 子舱位就是F
                routeInfoEntity.setCabin(baseCabin.equals("Y") ? "E" : "F");
                //routeInfoEntity.setCabinClass(routeInfoEntity.getCabin().equals("E") ? 1 : 3);
                routeInfoEntity.setCarrier(flyno.substring(0, 2));
                routeInfoEntity.setDepAirport(fromCity);
                routeInfoEntity.setDepTime((fromDate + deptime).replace("-", "").replace(":", "").replace(" ", ""));
                routeInfoEntity.setArrTime((arrdate + arrtime).replace("-", "").replace(":", "").replace(" ", ""));
                routeInfoEntity.setFlightNumber(flyno);
                if (flightSegment.getChildText("tripType").equals("OB")) {
                    fromSegments.add(routeInfoEntity);
                } else {
                    retSegments.add(routeInfoEntity);
                }
            }


            fromSegments.stream().sorted(Comparator.comparing(RouteInfoEntity::getDepTime));

            if (retSegments.size() != 0) {
                retSegments.stream().sorted(Comparator.comparing(RouteInfoEntity::getDepTime));
            }

            routings.setFromCity(fromSegments.get(0).getDepAirport());

            routings.setToCity(fromSegments.get(0).getArrAirport());

            routings.setFromDate(fromSegments.get(0).getDepTime());

            if (retSegments.size() != 0) {
                routings.setRetDate(retSegments.get(0).getDepTime());
            } else {
                routings.setRetDate("");
            }

            routings.setAdultPrice(price);

            routings.setTripType(tripType);

            routings.setFromSegments(fromSegments);
            routings.setRetSegments(retSegments);
            qunaerOrderParams.setRouting(routings);


        }
        return qunaerOrderParams;
    }


}
