package com.bjlt.spider.hangbanguanjia.service.impl;

import com.bjlt.spider.commonhandler.AirSoucePolicyForSearchHandler;
import com.bjlt.spider.commonhandler.AirSoucePolicyForVerifyHandler;
import com.bjlt.spider.common.PlatformCids;
import com.bjlt.spider.common.RedisKeyNames;
import com.bjlt.spider.common.SourceGroupIds;
import com.bjlt.spider.entity.*;
import com.bjlt.spider.hangbanguanjia.controller.HbgjBaseController;
import com.bjlt.spider.hangbanguanjia.entity.*;

import com.bjlt.spider.hangbanguanjia.service.HbgjSearchService;
import com.bjlt.spider.qunaer.entity.QunarRoutings;
import com.bjlt.spider.util.UUIDUtils;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static com.bjlt.spider.utils.CompareUtils.compareFlight;
import static com.bjlt.spider.utils.DateUtils.changeDateFormat;
import static com.bjlt.spider.utils.KeyUtils.genarateKey;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 18:19 2017/12/15
 */
@Slf4j
@Service
public class HbgjSearchServiceImpl extends HbgjBaseController implements HbgjSearchService {

    @Autowired
    private AirSoucePolicyForSearchHandler searchHandler;


    @Autowired
    private AirSoucePolicyForVerifyHandler verifyHandler;


    @Autowired
    private RedisTemplate<String, Object> redisTemplate;


    @Override
    public HbgjRespEntity search(HbgjEntity entity) throws Exception {

        List<QunarRoutings> routingsList = Lists.newArrayList();

        //判断航班类型
        HbgjTrip[] tripArray = entity.getTripList();

        List<HbgjTrip> tripList = Arrays.asList(tripArray);


        //不支持
        if (tripList.size() > 3) {
            return HbgjRespEntity.builder().msg("不支持三程以上").status(3).timestamp(System.currentTimeMillis())
                    .build();
        }


        if (tripList.size() >= 2){
            if (!tripList.get(0).getFromCity().equals(tripList.get(1).getToCity()) && !tripList.get(0).getToCity().equals(tripList.get(1).getFromCity())) {
                return HbgjRespEntity.builder().msg("不支持此查询方式").status(3).timestamp(System.currentTimeMillis())
                        .build();
            }
        }


        List<SearchResponseEntity> result = getResult(entity);


        if (result == null) {
            return HbgjRespEntity.builder().msg("程序异常").status(3).timestamp(System.currentTimeMillis())
                    .build();
        }
        routingsList = searchHandler.handlerList(result, PlatformCids.QUNA, null, String.valueOf(tripList.size()));

        return commonHandler(routingsList, tripList);

    }

    @Override
    public HbgjRespEntity verify(HbgjEntity entity) throws CloneNotSupportedException, IOException, ClassNotFoundException {


        HbgjTrip[] trips = entity.getTripList();

        List<HbgjTrip> tripList = Arrays.asList(trips);


        List<SearchResponseEntity> result = getResult(entity);

        String confirmPara = entity.getConfirmPara();


        QunarRoutings routings = (QunarRoutings) redisTemplate.opsForValue().get(confirmPara);


        SearchRequestAfterOta searchRequestAfterOta = new SearchRequestAfterOta();

        if (entity.getChildrenNum() != null) {
            searchRequestAfterOta.setChildNum(entity.getChildrenNum());
        }


        routings.setFromCity(tripList.get(0).getFromCity());

        routings.setToCity(tripList.get(0).getToCity());

        routings.setFromDate(tripList.get(0).getDate());

        String tripType = "";
        if (tripList.size() == 2) {
            routings.setRetDate(tripList.get(1).getDate());
            tripType = "2";
            routings.setTripType(tripType);
        } else {
            tripType = "1";
            routings.setTripType(tripType);
        }


        List<QunarRoutings> qunarRoutings = verifyHandler.handlerList(result, PlatformCids.QUNA, routings, tripType);


        return commonHandler(qunarRoutings, tripList);

    }


    public HbgjRespEntity commonHandler(List<QunarRoutings> routingsList, List<HbgjTrip> tripList) throws CloneNotSupportedException, IOException, ClassNotFoundException {
        if (routingsList == null || routingsList.size() == 0) {
            return HbgjRespEntity.builder().msg("无票").status(2).timestamp(System.currentTimeMillis())
                    .build();
        }


        AtomicInteger atomicInteger = new AtomicInteger();

        HbgjRespEntity respEntity = new HbgjRespEntity();

        List<MyFlight> myFlightList = new ArrayList<>();

        Set<String> keys = redisTemplate.keys(RedisKeyNames.FLIGHT + "*");

        for (String member : keys) {
            redisTemplate.delete(member);
        }


        for (QunarRoutings routings : routingsList) {

            List<Flight> flightList = new ArrayList<>();
            MyFlight myFlight = new MyFlight();

            List<RouteInfoEntity> from = routings.getFromSegments();

            List<RouteInfoEntity> rets = routings.getRetSegments();

            for (RouteInfoEntity fromSegment : from) {
                Flight flight = new Flight();
                flight.setFn(fromSegment.getFlightNumber());
                flight.setAircraftCode(fromSegment.getAircraftCode());
                flight.setOrg(fromSegment.getDepAirport());
                flight.setDst(fromSegment.getArrAirport());
                StopCityEntity stopCities = fromSegment.getStopCitiesEntity();
                Stop stop = new Stop();
                if (stopCities != null) {
                    BeanUtils.copyProperties(stopCities, stop);
                }
                List<Stop> stops = new ArrayList<>();
                stops.add(stop);
                flight.setStops(stops.toArray(new Stop[]{}));
                flight.setMeal("0");
                flight.setDt(changeDateFormat(fromSegment.getDepTime()));
                flight.setAt(changeDateFormat(fromSegment.getArrTime()));
                flight.setTripType("1");
                flightList.add(flight);

            }
            if (rets != null && rets.size() != 0) {
                for (RouteInfoEntity ret : rets) {
                    Flight flight = new Flight();
                    flight.setFn(ret.getFlightNumber());
                    flight.setAircraftCode(ret.getAircraftCode());
                    flight.setOrg(ret.getDepAirport());
                    flight.setDst(ret.getArrAirport());
                    StopCityEntity stopCities = ret.getStopCitiesEntity();
                    Stop stop = new Stop();
                    if (stopCities != null) {
                        BeanUtils.copyProperties(stopCities, stop);
                    }
                    List<Stop> stops = new ArrayList<>();
                    stops.add(stop);
                    flight.setStops(stops.toArray(new Stop[]{}));
                    flight.setMeal("0");
                    flight.setDt(changeDateFormat(ret.getDepTime()));
                    flight.setAt(changeDateFormat(ret.getArrTime()));

                    flight.setTripType("2");

                    flightList.add(flight);
                }
            }

            myFlight.setFlights(flightList);

            String key = RedisKeyNames.FLIGHT + genarateKey(myFlight.getFlights());


            redisTemplate.opsForSet().add(key, routings);

            //这个时候flightList 里面有四个元素 这是一个行程 判断这个行程在 list 里面有没有

            if (myFlightList.size() == 0) {
                myFlightList.add(myFlight);
            } else {
                boolean ret = compareFlight(myFlight, myFlightList);

                if (!ret) {
                    myFlightList.add(myFlight);
                }
            }

        }
        myFlightList.stream().forEach(t -> t.getFlights().stream().forEach(s -> s.setId(atomicInteger.getAndIncrement())));

        List<Solution> solutions = new ArrayList<>();


        for (MyFlight myFlight : myFlightList) {
            List<Flight> flights = myFlight.getFlights();

            String key = RedisKeyNames.FLIGHT + genarateKey(myFlight.getFlights());

            Set<Object> members = redisTemplate.opsForSet().members(key);

            Set<QunarRoutings> qunarRoutingsSet = new HashSet<>();

            for (Object member : members) {
                qunarRoutingsSet.add((QunarRoutings) member);
            }

            //现在拿到了价格  每一个都有多个价格 这个价格的个数是members的大小
            //每一个 Flight 都有一个 Flight 信息 两个 fare 信息

            Solution solution = new Solution();

            List<Fare> fareList = new ArrayList<>();

            for (QunarRoutings routings : qunarRoutingsSet) {


                Fare fare = new Fare();

                fare.setTags(1);
                fare.setCurrency(routings.getCurreny());
                fare.setAdultPrice(routings.getAdultPrice().intValue());
                fare.setAdultTax(routings.getAdultTax().intValue());
                fare.setChildPrice(routings.getChildPrice().intValue());
                fare.setChildTax(routings.getChildTax().intValue());
                fare.setProductType("COMMON");
                fare.setTicketTime(routings.getTicketTimeLimit());


                String confirmPara = RedisKeyNames.CONFIRMPARA + UUIDUtils.getUUID();

                fare.setConfirmPara(confirmPara);

                redisTemplate.opsForValue().set(confirmPara, routings, 1, TimeUnit.HOURS);

                Restriction restriction = Restriction.builder()
                        .maxAge(99)
                        .minAge(0)
                        .minPassenger(1)
                        .maxPassenger(9)
                        .build();


                fare.setRestriction(restriction);


                List<Cabin> cabinList = new ArrayList<>();

                List<RouteInfoEntity> fromSegments = routings.getFromSegments();


                for (RouteInfoEntity from : fromSegments) {

                    Cabin cabin = new Cabin();
                    if (from.getCabinClass().equals(3)) {
                        cabin.setBct("F");
                        cabin.setCt("C");
                    } else {
                        cabin.setBct("Y");
                        cabin.setCt("E");
                    }
                    cabin.setSeq(0);

                    cabin.setLeft(String.valueOf(routings.getMaxSeats()));

                    cabinList.add(cabin);
                }


                List<RouteInfoEntity> retSegments = routings.getRetSegments();

                if (retSegments != null && retSegments.size() != 0) {
                    for (RouteInfoEntity retSegment : retSegments) {
                        Cabin cabin = new Cabin();
                        if (retSegment.getCabinClass().equals(3)) {
                            cabin.setBct("F");
                            cabin.setCt("C");
                        } else {
                            cabin.setBct("Y");
                            cabin.setCt("E");
                        }
                        cabin.setSeq(1);
                        cabinList.add(cabin);
                        cabin.setLeft(String.valueOf(routings.getMaxSeats()));
                    }
                }


                fareList.add(fare);
                fare.setCabins(cabinList.toArray(new Cabin[]{}));
            }


            solution.setFares(fareList.toArray(new Fare[]{}));

            List<Trip> trips = new ArrayList<>();

            List<Integer> integerList = new ArrayList<>();

            List<Integer> integerListR = new ArrayList<>();
            if (tripList.size() == 1) {
                //只有一个 trip 节点
                Trip trip = new Trip();

                trip.setDate(tripList.get(0).getDate());
                trip.setFromCity(tripList.get(0).getFromCity());
                trip.setToCity(tripList.get(0).getToCity());

                for (Flight flight : flights) {
                    integerList.add(flight.getId());
                    trip.setFlights(integerList.toArray(new Integer[]{}));

                }
                trips.add(trip);
            } else {
                Trip trip = new Trip();

                trip.setDate(tripList.get(0).getDate());
                trip.setFromCity(tripList.get(0).getFromCity());
                trip.setToCity(tripList.get(0).getToCity());

                Trip tripR = new Trip();

                tripR.setDate(tripList.get(1).getDate());
                tripR.setFromCity(tripList.get(1).getFromCity());
                tripR.setToCity(tripList.get(1).getToCity());


                for (Flight flight : flights) {
                    if (flight.getTripType().equals("1")) {
                        integerList.add(flight.getId());
                    }
                    if (flight.getTripType().equals("2")) {
                        integerListR.add(flight.getId());
                    }
                }
                trip.setFlights(integerList.toArray(new Integer[]{}));
                tripR.setFlights(integerListR.toArray(new Integer[]{}));

                trips.add(trip);

                trips.add(tripR);
            }

            solution.setRequestTrips(trips.toArray(new Trip[]{}));
            solutions.add(solution);
        }
        List<List<Flight>> collect = myFlightList.stream().map(t -> t.getFlights()).collect(Collectors.toList());

        List<Flight> flightList = new ArrayList<>();

        for (List<Flight> list : collect) {
            flightList.addAll(list);
        }


        //最后退改签和行李额
        List<ScRefundHbgjPenalty> scRefundHbgjPenalties = (List<ScRefundHbgjPenalty>) redisTemplate.opsForValue().get(RedisKeyNames.REFUND+PlatformCids.HBGJ);


        if (scRefundHbgjPenalties != null && scRefundHbgjPenalties.size() != 0) {
            scRefundHbgjPenalties = scRefundHbgjPenalties.stream().filter(t -> t.getCid().equals(PlatformCids.HBGJ) && t.getSourceGroupId().equals(SourceGroupIds.AIRASIA)).collect(Collectors.toList());
        }


        List<ScHbgjCharge> scHbgjCharges = (List<ScHbgjCharge>) redisTemplate.opsForValue().get(RedisKeyNames.CHARGE+PlatformCids.HBGJ);


        if (scRefundHbgjPenalties != null && scRefundHbgjPenalties.size() >= 1) {


            ScRefundHbgjPenalty scRefundHbgjPenalty = scRefundHbgjPenalties.get(0);


            Penalty penalty = new Penalty();

            Rule changeRule = new Rule();


            Rule refundRule = new Rule();

            Rule noShowRule = new Rule();


            Integer isChange = scRefundHbgjPenalty.getIsChange();

            if (isChange.equals(0)) {
                penalty.setChange(null);
            } else {
                String changeRuleId = scRefundHbgjPenalty.getChangeRuleId();

                List<ScHbgjCharge> collect1 = scHbgjCharges.stream().filter(t -> t.getRelateid().equals(changeRuleId)).collect(Collectors.toList());

                insetTGQ(collect1, changeRule);
            }

            penalty.setChange(changeRule);

            Integer isRefund = scRefundHbgjPenalty.getIsRefund();


            if (isRefund.equals(0)) {
                penalty.setRefund(null);
            } else {
                String refundRuleId = scRefundHbgjPenalty.getRefundRuleId();

                List<ScHbgjCharge> collect1 = scHbgjCharges.stream().filter(t -> t.getRelateid().equals(refundRuleId)).collect(Collectors.toList());

                insetTGQ(collect1, refundRule);
            }

            penalty.setRefund(refundRule);

            Integer isNoshow = scRefundHbgjPenalty.getIsNoshow();

            if (isNoshow.equals(0)) {
                penalty.setNoshow(null);
            } else {
                String noshowRuleId = scRefundHbgjPenalty.getNoshowRuleId();

                List<ScHbgjCharge> collect1 = scHbgjCharges.stream().filter(t -> t.getRelateid().equals(noshowRuleId)).collect(Collectors.toList());


                insetTGQ(collect1, noShowRule);
            }

            penalty.setRefund(noShowRule);


            Integer isBaggage = scRefundHbgjPenalty.getIsBaggage();

            Baggage baggage = new Baggage();
            if (isBaggage.equals(0)) {
                baggage.setAllowedPieces(0);
                baggage.setAllowedWeight(0);
            } else {
                baggage.setAllowedPieces(scRefundHbgjPenalty.getBaggagePieces());
                baggage.setAllowedWeight(scRefundHbgjPenalty.getBaggageWeight());
            }


            solutions.stream().forEach(t -> {

                Fare[] fares = t.getFares();
                for (Fare fare : fares) {
                    fare.setAdultPenalty(penalty);
                    fare.setChildPenalty(penalty);

                    Arrays.stream(fare.getCabins()).forEach(s -> {
                        s.setAdultBaggage(baggage);
                        s.setChildBaggage(baggage);
                    });
                }

            });
        }

        respEntity.setSolutions(solutions.toArray(new Solution[]{}));
        respEntity.setFlights(flightList.toArray(new Flight[]{}));

        respEntity.setStatus(0);
        respEntity.setMsg("OK");
        respEntity.setTimestamp(System.currentTimeMillis());

        log.info("----------处理完毕----------");

        return respEntity;
    }


    private void insetTGQ(List<ScHbgjCharge> collect1, Rule rule) {

        List<Charge> Charges = new ArrayList<>();
        if (collect1 != null && collect1.size() != 0) {
            ScHbgjCharge scHbgjCharge = collect1.get(0);


            rule.setDesc(scHbgjCharge.getDesc());

            rule.setServiceCharge(scHbgjCharge.getServiceCharge());


            for (ScHbgjCharge hbgjCharge : collect1) {
                Charge charge = new Charge();

                charge.setFrom(Integer.parseInt(hbgjCharge.getFrom()));

                charge.setTo(Integer.parseInt(hbgjCharge.getTo()));

                charge.setFee(Integer.parseInt(hbgjCharge.getFee()));

                Charges.add(charge);
            }


        }
        rule.setCharges(Charges.toArray(new Charge[]{}));

    }


}
