package com.bjlt.spider.qunaer.cotroller;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.common.MethedNames;
import com.bjlt.spider.entity.PlatformSourceVo;
import com.bjlt.spider.qunaer.entity.*;
import com.bjlt.spider.qunaer.service.QunaerService;
import com.bjlt.spider.util.RedisPrefix;
import com.bjlt.spider.util.UUIDUtils;
import com.bjlt.spider.utils.LoadParam;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 15:53 2017/10/25
 */
@RestController
@RequestMapping(value = "/qunaer")
@Slf4j
public class QunaerSearchAndVerifyController {


    @Autowired
    private QunaerService qunaerService;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;


    @PostMapping(value = "search", consumes = MediaType.TEXT_PLAIN_VALUE)
    public QunaerSearchRespEntity search(HttpServletRequest request) throws Exception {

        String param = LoadParam.read(request.getInputStream());
        JSONObject object = JSONObject.parseObject(param);
        String fromDate = object.getString("fromDate");
        String fromCity = object.getString("fromCity");
        String toCity = object.getString("toCity");
        String retDate = object.getString("retDate");
        String cid = object.getString("cid");
        Integer adultNum = object.getInteger("adultNum");
        Integer childNum = object.getInteger("childNum");

        String tripType = object.getString("tripType");

        if (StringUtils.isBlank(fromCity) || StringUtils.isBlank(fromDate) || StringUtils.isBlank(toCity)
                || StringUtils.isBlank(cid) || adultNum == null) {
            return QunaerSearchRespEntity.error("参数错误");
        }

        if (childNum == null) {
            childNum = 0;
        }
        if (adultNum == null) {
            adultNum = 0;
        }
        if (tripType.equals("2") && StringUtils.isBlank(retDate)) {
            return QunaerSearchRespEntity.error("参数错误");
        }
        Integer passengerCount = new Integer(0);


        //拿到默认人数
        //TODO 大 BUG
        PlatformSourceVo vo = (PlatformSourceVo) redisTemplate.opsForValue().get(RedisPrefix.AIRSOURCEPLATFORM.getPrefix() + cid + "1");

        Integer defultCount = null;
        if (vo != null) {
            defultCount = vo.getPassengerCount();
        } else {
            defultCount = 3;
        }


        if (adultNum + childNum <= defultCount) {
            passengerCount = defultCount;
        } else {
            passengerCount = adultNum + childNum;
        }


        QunaerSearchParams params = QunaerSearchParams.builder()
                .adultNum(passengerCount)
                .childNum(0)
                .cid(cid)
                .fromCity(fromCity)
                .retDate(retDate)
                .tripType(tripType)
                .fromDate(fromDate)
                .toCity(toCity)
                .build();
        return qunaerService.search(params);
    }


    @PostMapping(value = "verify", consumes = MediaType.TEXT_PLAIN_VALUE)
    public QunaerVerifyRespEntity verify(HttpServletRequest request) throws Exception {


        String param = LoadParam.read(request.getInputStream());

        log.info("QUNAER 请求验价参数是{}", param);

        String uuid = UUIDUtils.getUUID();


        JSONObject object = JSONObject.parseObject(param);

        //redisTemplate.opsForValue().set(MethedNames.verify + uuid, param);

        String cid = object.getString("cid");
        Integer adultNum = object.getInteger("adultNum");
        Integer childNum = object.getInteger("childNum");

        String tripType = object.getString("tripType");

        if (childNum == null) {
            childNum = 0;
        }

        QunaerVerifyParams params = new QunaerVerifyParams();

        params.setCid(cid);

        params.setTripType(tripType);

        params.setChildNum(childNum);

        params.setAdultNum(adultNum);

        String routingString = object.getJSONObject("routing").toJSONString();

        ObjectMapper objectMapper = new ObjectMapper();

        QunarRoutings qunarRoutings = objectMapper.readValue(routingString, QunarRoutings.class);

        params.setRouting(qunarRoutings);


        if (params.getChildNum() == null) {
            params.setChildNum(0);
        }
        log.info("收到来自去哪儿平台的 verify 请求{} ", params.toString());

        QunaerVerifyRespEntity verifyRespEntity = qunaerService.verify(params, qunarRoutings.getData());


        return verifyRespEntity;
    }
}
