package com.bjlt.spider.qunaer.entity;

import com.bjlt.spider.policy.entity.Page;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.math.BigDecimal;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 15:33 2017/10/25
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class QunaerRule  extends Page implements Serializable{
    private static final long serialVersionUID = -4105728797405985372L;

    private Integer id;

    /**
     * 航司
     */
    @JsonIgnore
    private Integer sourceGroupId;

    /**
     * 仓位等级
     */
    @JsonIgnore
    private Integer tag;

    /**
     *  是否允许退票 ：  0 - 表示不允许 ， 1 - 表示允许
     */
    private  Integer hasRefund;

    /**
     * 可输入格式如：200-72-300-48-1000-0-*，代表72小时前退票手续费200（单位以输入的币种为准）；
     * 48小时到72小时之间，退票手续费300（单位以输入的币种为准）；
     * 飞机起飞不足48小时，手续费1000（单位以输入的币种为准）；起飞后不予退票（输入*），为空表示按航司规定
     */
    private  String  refund;

    /**
     * 部分退票规则 0 - 表示不允许， 1- 表示允许 ；单程直飞为 0
     */
    private Integer partRefund;


    /**
     * 退票服务费
     */
    private BigDecimal refundServiceCharge;



    /**
     * 1：部分退票费用，为空或正整数，为空表示按航司规定执行（单位以所输入的币种为准，如：USD, EUR, 或IDR等）
     *  2： 当currency字段输入不合法，或输入不为支付中心支持的币种符号，该字段不返回结果
     *
     */
    private Integer partRefundPrice;


    /**
     * 是否允许改期 ： 0 - 不允许， 1- 允许
     */
    private  Integer hasEndorse;


    /**
     * 可输入格式如：200-72-300-48-1000-0-*，
     * 代表72小时前改期手续费200（单位以输入的币种为准）；
     * 48小时到72小时之间，改期手续费300（单位以输入的币种为准）；
     * 飞机起飞不足48小时，手续费1000（单位以输入的币种为准）；
     * 起飞后不予改期（输入*），为空表示按航司规定
     */
    private  String  endorse;

    /**
     * 部分改期规则，0 - 表示不允许，1 - 表示允许；单程直飞为0
     */
    private  Integer partEndorse;

    /**
     * 1：部分改期费用，为空或正整数，为空表示按航司规定执行（单位以所输入的币种为准，如：USD, EUR, 或IDR等）
     *     2： 当currency字段输入不合法，或输入不为支付中心支持的币种符号，该字段不返回结果
     */
    private  Integer partEndorsePrice;


    /**
     * 签转规则，0 - 表示不支持，1 - 表示支持
     */
    private  Integer endorsement;

    /**
     * 是否提供免费行李额：0 - 表示不提供，1 - 表示提供
     */
    private  Integer hasBaggage;


    /**
     * 单程直飞时，样例：1-23，表示1PC，23kg；单程中转和往返直飞录入1-23;1-23，
     * 中间用分号隔开；往返中转录入1-23;1-23;1-23;1-23。
     * 如某段为空表示按航空公司规定执行，如“-;-;-;-”表示四段均按航司规定执行（其中分号不可缺少）
     */
    private  String baggage;


    /**
     * 是否有noshow规则，
     * 0表示没有规则限制，
     * 1表示有规则限制
     */
    private  Integer hasNoShow;


    /**
     * Noshow时限，输入正整数
     */
    private  Integer noShowLimitTime;


    /**
     * 1：Noshow罚金，可为空，若输入则为正整数。输入为空表示按照航司规定执行（单位以所输入的币种为准，如：USD, EUR, 或IDR等）
     * 2： 当currency字段输入不合法，或输入不为支付中心支持的币种符号，该字段不返回结果
     */
    private Integer penalty;

    /**
     * Noshow特殊规则：
     * 0（无） 1（Noshow时限内不允许退票）
     * 2（Noshow时限内不允许改期）
     * 3（Noshow时限内不允许退票和改期）
     */
    private   Integer specialNoShow;

    /**
     * 其他说明，最大 300 个字符
     */
    private  String  other;


    private String editUser;

    /**
     添加时间
     */
    private String addTime;

    /**
     更新时间
     */
    private String updateTime;

    /**
     * 状态  0 无效 1 有效
     */
    private Integer status;




}
