package com.bjlt.spider.qunaer.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.common.DataEntity;
import com.bjlt.spider.common.MethedNames;
import com.bjlt.spider.commonhandler.AirSoucePolicyForOrderHandler;
import com.bjlt.spider.commonhandler.AirSoucePolicyForSearchHandler;
import com.bjlt.spider.commonhandler.AirSoucePolicyForVerifyHandler;
import com.bjlt.spider.entity.*;
import com.bjlt.spider.interfaces.ControRpc;
import com.bjlt.spider.orderInfo.service.OrOrdertbService;
import com.bjlt.spider.qunaer.entity.PassengersInfo;
import com.bjlt.spider.qunaer.entity.*;
import com.bjlt.spider.qunaer.service.QunaerService;
import com.bjlt.spider.util.RespEntity;
import com.bjlt.spider.util.UUIDUtils;
import com.bjlt.spider.utils.*;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.math.BigDecimal;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 16:11 2017/10/25
 */
@Slf4j
@Service
public class QunaerServiceImpl implements QunaerService {


    @Autowired
    private ControRpc controRpc;


    @Autowired
    private AirSoucePolicyForSearchHandler searchHandler;

    @Autowired
    private AirSoucePolicyForOrderHandler orderHandler;

    @Autowired
    private AirSoucePolicyForVerifyHandler verifyHandler;


    @Autowired
    private OrOrdertbService orOrdertbService;


    private ObjectMapper objectMapper = new ObjectMapper();

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;


    @Override
    public QunaerSearchRespEntity search(QunaerSearchParams searchParams) throws Exception {

        List<QunarRoutings> routingsList = Lists.newArrayList();
        List<SearchResponseEntity> responseEntityArrayList = Lists.newArrayList();


        //获取数据结果
        SearchRequestAfterOta ota = SearchRequestAfterOta.builder()
                .adultNum(Math.abs(searchParams.getAdultNum()))
                .childNum(Math.abs(searchParams.getChildNum()))
                .fromCity(searchParams.getFromCity())
                .toCity(searchParams.getToCity())
                .retDate(searchParams.getRetDate())
                .fromDate(searchParams.getFromDate())
                .cid(searchParams.getCid())
                .tripType(searchParams.getTripType())
                .methodName(MethedNames.search)
                .cache(true)
                .build();

        String uuid = "";
        if (searchParams.getTripType().equals("1")) {
            uuid = OtaConfigGenerateKey.fromKey(ota);
        } else {
            uuid = OtaConfigGenerateKey.allKey(ota);
        }
        ota.setUuid(uuid);

        try {
            log.info("开始向控制中心发送search请求{}", searchParams.toString());
            RespEntity spiderEntity = controRpc.createSpider(ota);


            if (spiderEntity.getStatus() != 200) {
                return QunaerSearchRespEntity.error("search failed");
            }


            Object attachment = spiderEntity.getAttachment();

            List retList = (List) attachment;


            for (Object o : retList) {
                String searchEntityString = JSONObject.toJSONString(o);


                try {
                    SearchResponseEntity responseEntity = objectMapper.readValue(searchEntityString, SearchResponseEntity.class);


                    if ((responseEntity.getUuid()).equals(ota.getUuid())) {
                        responseEntityArrayList.add(responseEntity);
                    }


                } catch (IOException e) {
                    e.printStackTrace();
                }
            }


            if (responseEntityArrayList.size() == 0) {
                return QunaerSearchRespEntity.error("search failed");
            }

            Object o = JSONObject.toJSON(responseEntityArrayList);
            //log.info("---------拿到搜索服务的结果{}---------", o);


            routingsList = searchHandler.handlerList(responseEntityArrayList, searchParams.getCid(), null, searchParams.getTripType());

            if (searchParams.getTripType().equals("2")) {
                routingsList = routingsList.stream().filter(t -> t.getFromSegments().size() == t.getRetSegments().size()).collect(Collectors.toList());
            }


            routingsList.stream().forEach(t -> t.setCurreny("CNY"));
            routingsList.stream().forEach(t -> t.setFromCity(searchParams.getFromCity()));
            routingsList.stream().forEach(t -> t.setToCity(searchParams.getToCity()));


            List<QunarRoutings> finalroutingsList = new ArrayList<>();

            if (searchParams.getTripType().equals("2")) {
                Map<Map<List<RouteInfoEntity>, List<RouteInfoEntity>>, QunarRoutings> map = new HashMap<>();

                for (QunarRoutings qunarRoutings : routingsList) {
                    List<RouteInfoEntity> fromSegments = qunarRoutings.getFromSegments();
                    List<RouteInfoEntity> retSegments = qunarRoutings.getRetSegments();
                    Map<List<RouteInfoEntity>, List<RouteInfoEntity>> listMap = new HashMap<>();
                    listMap.put(fromSegments, retSegments);
                    QunarRoutings qunarRoutings1 = map.get(listMap);
                    if (qunarRoutings1 == null || !qunarRoutings.getTag().equals(qunarRoutings1.getTag())) {
                        map.put(listMap, qunarRoutings);
                    } else if (qunarRoutings1.getAdultPrice().compareTo(qunarRoutings.getAdultPrice()) > 0) {
                        map.remove(listMap);
                        map.put(listMap, qunarRoutings);
                    }
                }
                finalroutingsList = new ArrayList<>(map.values());
            } else {
                finalroutingsList = routingsList;
            }
            //Set<String> keysFrom = redisTemplate.keys(RedisPrefix.AIRLINEEXCHANGE.getPrefix() + searchParams.getFromCity());

            //Set<String> keysTo = redisTemplate.keys(RedisPrefix.AIRLINEEXCHANGE.getPrefix() + searchParams.getToCity());

//            routingsList.stream().forEach(t->{
//                t.getFromSegments().stream().forEach(t->t.get);
//            });

            return QunaerSearchRespEntity.success(finalroutingsList, "OK");
        } catch (Exception e) {
            e.printStackTrace();
            return QunaerSearchRespEntity.error("search failed");
        }

    }

    @Override
    public QunaerVerifyRespEntity verify(QunaerVerifyParams verifyParams, String data) throws Exception {
        List<SearchResponseEntity> responseEntityArrayList = Lists.newArrayList();
        QunaerVerifyRespEntity verifyRespEntity = new QunaerVerifyRespEntity();


        BigDecimal price = new BigDecimal(0);


        DataEntity entity = CreateDataUtils.getinfo(verifyParams.getRouting().getData());

        String fromCity = entity.getFromCity();
        String toCity = entity.getToCity();
        String fromDate = entity.getFromDate();
        String retDate = entity.getRetDate();

        String sourceId = entity.getSourceId();
        PriceMap priceMap = entity.getPriceMap();


        SearchRequestAfterOta searchRequestAfterOta = new SearchRequestAfterOta();

        QunarRoutings routings = verifyParams.getRouting();

        if (verifyParams.getChildNum() != null) {
            searchRequestAfterOta.setChildNum(Integer.valueOf(verifyParams.getChildNum()));
        }
        routings.getFromSegments().stream().forEach(t -> t.setCabinClass(null));

        if (verifyParams.getTripType().equals("2")) {
            routings.getRetSegments().stream().forEach(t -> t.setCabinClass(null));

        }

        routings.setPriceMap(priceMap);

        routings.setFromCity(fromCity);

        routings.setToCity(toCity);

        routings.setFromDate(fromDate);

        routings.setRetDate(retDate);

        routings.setAdultPrice(price);

        routings.setTripType(verifyParams.getTripType());

        searchRequestAfterOta.setSourceId(sourceId);

        searchRequestAfterOta.setAdultNum(Integer.valueOf(verifyParams.getAdultNum()));

        searchRequestAfterOta.setCid(verifyParams.getCid());

        searchRequestAfterOta.setFromCity(fromCity);

        searchRequestAfterOta.setToCity(toCity);

        searchRequestAfterOta.setFromDate(fromDate.replace("-", ""));

        searchRequestAfterOta.setRetDate(retDate.replace("-", ""));

        searchRequestAfterOta.setTripType(verifyParams.getTripType());

        searchRequestAfterOta.setMethodName(MethedNames.verify);

        String uuid = "";
        if (verifyParams.getTripType().equals("1")) {
            uuid = OtaConfigGenerateKey.fromKey(searchRequestAfterOta);
        } else {
            uuid = OtaConfigGenerateKey.allKey(searchRequestAfterOta);
        }
        searchRequestAfterOta.setUuid(uuid);

        searchRequestAfterOta.setCache(false);

        RespEntity spiderEntity = null;
        //现在去搜索
        try {
            spiderEntity = controRpc.createSpider(searchRequestAfterOta);
        } catch (SQLException e) {
            e.printStackTrace();
        }


        if (spiderEntity.getStatus() != 200) {
            return QunaerVerifyRespEntity.error("verify failed");
        }


        Object attachment = spiderEntity.getAttachment();

        List retList = (List) attachment;


        if (retList.size() == 0) {
            return QunaerVerifyRespEntity.error("verify failed");
        }

        for (Object o : retList) {
            String searchEntityString = JSONObject.toJSONString(o);


            try {
                SearchResponseEntity responseEntity = objectMapper.readValue(searchEntityString, SearchResponseEntity.class);

                String key = searchRequestAfterOta.getUuid();

                if (responseEntity.getUuid().equals(key)) {
                    responseEntityArrayList.add(responseEntity);
                }


            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        Object o = JSONObject.toJSON(responseEntityArrayList);
        // log.info("---------拿到验价服务的结果{}---------", o);
        List<QunarRoutings> qunarRoutings = verifyHandler.handlerList(responseEntityArrayList, verifyParams.getCid(), routings, verifyParams.getTripType());

        if (qunarRoutings == null || qunarRoutings.size() == 0) {
            return QunaerVerifyRespEntity.error("verify failed");
        }
        QunarRoutings qunarRoutings1 = qunarRoutings.get(0);


        verifyRespEntity.setRule(qunarRoutings1.getRule());
        qunarRoutings1.setRule(null);
        verifyRespEntity.setSessionId(UUIDUtils.getUUID());


        qunarRoutings.get(0).setData(data);

        verifyRespEntity.setMsg("success");
        verifyRespEntity.setRouting(qunarRoutings.get(0));
        verifyRespEntity.setMaxSeats(qunarRoutings.get(0).getMaxSeats());


        String s = JSONObject.toJSONString(verifyRespEntity);
        log.info("V返回结果{}", s);
        return verifyRespEntity;

    }


    @Override
    public QunaerOrderRespEntity order(QunaerOrderParams orderParams, String source) throws IOException {

        List<SearchResponseEntity> responseEntityArrayList = Lists.newArrayList();
        QunaerOrderRespEntity orderRespEntity = new QunaerOrderRespEntity();

        //通过data 解析出航班信息
        DataEntity data = CreateDataUtils.getinfo(orderParams.getRouting().getData());

        String fromDate = data.getFromDate();
        String retDate = data.getRetDate();
        String fromCity = data.getFromCity();
        String toCity = data.getToCity();
        PriceMap priceMap = data.getPriceMap();

        String pid = data.getPolicyId();
        BigDecimal price = new BigDecimal(0);
        String sourceId = data.getSourceId();

        //String routingString = redisTemplate.opsForValue().get();

        QunarRoutings routings = orderParams.getRouting();


        SearchRequestAfterOta searchRequestAfterOta = new SearchRequestAfterOta();

        routings.setFromCity(fromCity);

        routings.setToCity(toCity);

        routings.setFromDate(fromDate);

        routings.setRetDate(retDate);

        routings.setAdultPrice(price);

        routings.setTripType(orderParams.getTripType());


        routings.setPriceMap(priceMap);

        searchRequestAfterOta.setAdultNum(orderParams.getPassengers().length);

        searchRequestAfterOta.setCid(orderParams.getCid());

        searchRequestAfterOta.setFromCity(fromCity);

        searchRequestAfterOta.setToCity(toCity);

        searchRequestAfterOta.setMethodName(MethedNames.order);

        searchRequestAfterOta.setSourceId(sourceId);
        searchRequestAfterOta.setFromDate(fromDate.replace("-", "").substring(0, 8));

        if (!retDate.equals("")) {
            searchRequestAfterOta.setRetDate(retDate.replace("-", "").substring(0, 8));

        }


        searchRequestAfterOta.setTripType(orderParams.getTripType());


        String uuid = "";
        if (orderParams.getTripType().equals("1")) {
            uuid = OtaConfigGenerateKey.fromKey(searchRequestAfterOta);
        } else {
            uuid = OtaConfigGenerateKey.allKey(searchRequestAfterOta);
        }

        searchRequestAfterOta.setUuid(uuid);
        searchRequestAfterOta.setCache(false);

        RespEntity spiderEntity = null;
        //现在去搜索
        try {
            spiderEntity = controRpc.createSpider(searchRequestAfterOta);
        } catch (SQLException e) {
            e.printStackTrace();
        }


        if (spiderEntity.getStatus() != 200) {
            return QunaerOrderRespEntity.error("verify failed");
        }


        Object attachment = spiderEntity.getAttachment();

        List retList = (List) attachment;


        if (retList.size() == 0) {
            return QunaerOrderRespEntity.error("orderInfo failed");
        }
        log.info("---------拿到搜索服务的结果---------");

        retList.forEach(t -> {

            String searchEntityString = JSONObject.toJSONString(t);
            try {
                SearchResponseEntity responseEntity = objectMapper.readValue(searchEntityString, SearchResponseEntity.class);

                if (responseEntity.getUuid().equals(searchRequestAfterOta.getUuid())) {
                    responseEntityArrayList.add(responseEntity);
                }


            } catch (IOException e) {
                e.printStackTrace();
            }

        });
        List<QunarRoutings> qunarRoutings = orderHandler.handlerList(responseEntityArrayList, orderParams.getCid(), routings, searchRequestAfterOta.getTripType());

        if (qunarRoutings == null || qunarRoutings.size() == 0) {
            return QunaerOrderRespEntity.error("orderInfo failed");
        }
        orderParams.setSessionId(UUIDUtils.getUUID());
        qunarRoutings.get(0).setData(orderParams.getRouting().getData());
        orderRespEntity.setRouting(qunarRoutings.get(0));
        String pnr = PnrUtils.getPnr();
        orderRespEntity.setPnrCode(pnr);

        orderParams.setRouting(qunarRoutings.get(0));

        BigDecimal allPrice = new BigDecimal(orderParams.getPassengers().length).multiply(qunarRoutings.get(0).getAdultPrice());
        //入库

        // amqpTemplate.convertSendAndReceive()


        String name = "";
        String bir = "";
        String gender = "";
        String card = "";
        String cardType = "";
        String countryType = "";
        if (orderParams.getPassengers().length != 1) {
            for (PassengersInfo passengersInfo : orderParams.getPassengers()) {
                name += passengersInfo.getName() + "&";
                bir += passengersInfo.getBirthday() + "&";
                gender += passengersInfo.getGender() + "&";
                card += passengersInfo.getCardNum() + "&";
                cardType += passengersInfo.getCardType() + "&";
                countryType += passengersInfo.getNationality() + "&";
            }
            name = name.substring(0, name.length() - 1);
            bir = bir.substring(0, bir.length() - 1);

            gender = gender.substring(0, gender.length() - 1);

            cardType = cardType.substring(0, cardType.length() - 1);

            countryType = countryType.substring(0, countryType.length() - 1);


        } else {
            name = orderParams.getPassengers()[0].getName();
            bir = orderParams.getPassengers()[0].getBirthday();
            gender = orderParams.getPassengers()[0].getGender();
            card = orderParams.getPassengers()[0].getCardNum();

            cardType = orderParams.getPassengers()[0].getCardType();

            countryType = orderParams.getPassengers()[0].getNationality();


        }


        String orderId = ActionIdUtils.getUUID();
        OrOrdertb orOrdertb = OrOrdertb.builder()

                .ordersource(source)
                .orderno(orderId)
                .flighttype(orderParams.getTripType())
                .ordercodestatus(OrderCodeStatus.ORDER_INIT.getStatus())
                .orderstatus(OrderStatus.ORDER_INIT.getStatus())
                .airlinecount(qunarRoutings.get(0).getFromSegments().size())
                .childrencount((int) Arrays.stream(orderParams.getPassengers()).filter(t -> t.getAgeType().equals("1")).count())
                .adultscount((int) Arrays.stream(orderParams.getPassengers()).filter(t -> t.getAgeType().equals("0")).count())
                .passengercount(orderParams.getPassengers().length)
                .adultprice(qunarRoutings.get(0).getAdultPrice())
                .adulttax(qunarRoutings.get(0).getAdultTax())
                .childtax(qunarRoutings.get(0).getChildTax())
                .childprice(qunarRoutings.get(0).getChildPrice())
                .allprice(allPrice)
                .passengerNanation(countryType)
                .passengerCardtype(cardType)
                .nopay(allPrice)
                .pnrcode(pnr)
                .policyid(pid)
                .passengerName(name)
                .passengerBirth(bir)
                .passengerCardnum(card)
                .passengerGender(gender)
                .receiver(orderParams.getContact().getName())
                .postcode(orderParams.getContact().getPostcode())
                .address(orderParams.getContact().getAddress())
                .createtime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()))
                .lastupdated(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()))
                .operator("System")
                .build();
        orOrdertbService.insertSelective(orOrdertb);

        orderParams.setOrderID(orderId);


        orderRespEntity.setMsg("success");
        orderRespEntity.setOrderNo(orderId);

        orderRespEntity.setSessionId(orderParams.getSessionId());

        orderRespEntity.setMaxSeats(qunarRoutings.get(0).getMaxSeats());

        orderRespEntity.setStatus(0);

        orderRespEntity.setFee(qunarRoutings.get(0).getAdultPrice().multiply(BigDecimal.valueOf(orderParams.getPassengers().length)));


        try {
            new Thread(new OrderUtils(redisTemplate, orOrdertbService, orderParams),"ORDER").start();
        } catch (Exception e) {
            log.info("自动生单失败");
            return orderRespEntity;
        }
        return orderRespEntity;

    }


    public static void main(String[] args) {
        CreateDataUtils.getinfo("eyJmcm9tQ2l0eSI6IkNBTiIsImZyb21EYXRlIjoiMjAxOC0wMy0yMSIsInBvbGljeUlkIjoibnVsbCIsInByaWNlTWFwIjp7ImFkdWx0UHJpY2UiOjk4MiwiYWR1bHRUYXgiOjEwOSwiY2FiaW4iOiJFQyIsImNoaWxkUHJpY2UiOjEwODIsImNoaWxkVGF4IjoxMDksImZyb21UYWciOjAsIm1heFNlYXRzIjo1fSwicmV0RGF0ZSI6IiIsInRvQ2l0eSI6IlRXVSJ9");
    }
}



