package com.bjlt.spider.taobao.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.taobao.entity.*;
import com.bjlt.spider.taobao.service.TaobaoOrderService;
import com.bjlt.spider.utils.Encrypt;
import com.bjlt.spider.utils.LoadParam;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 14:21 2018/1/16
 */
@Controller
@RequestMapping(value = "/taobao")
@Slf4j
public class OrderController {


    @Autowired
    private TaobaoOrderService orderService;

    private ObjectMapper objectMapper = new ObjectMapper();

    @ResponseBody
    @PostMapping(value = "order", consumes = MediaType.TEXT_PLAIN_VALUE)
    public String order(HttpServletRequest request) throws Exception {

        log.info("---------来自 TAOBAO 的 order 请求---------");


        String param = LoadParam.read(request.getInputStream());

        String decrypt = Encrypt.decrypt(param, "Ir9ym6Bc8lCrrNvC");

        JSONObject object = JSONObject.parseObject(decrypt);

        String cid = object.getString("cid");

        String tripType = object.getString("tripType");

        String sessionId = object.getString("sessionId");

        String routingString = object.getJSONObject("routing").toJSONString();


        TaobaoRouting taobaoRouting = objectMapper.readValue(routingString, TaobaoRouting.class);

        List<TaobaoPassenger> taobaoPassengerList = Lists.newArrayList();

        JSONArray passengers = object.getJSONArray("passengers");

        for (Object passenger : passengers) {

            TaobaoPassenger taobaoPassenger = objectMapper.readValue(passenger.toString(), TaobaoPassenger.class);

            taobaoPassengerList.add(taobaoPassenger);
        }

        String contactString = object.getJSONObject("contact").toJSONString();

        TaobaoContact taobaoContact = objectMapper.readValue(contactString, TaobaoContact.class);


        JSONArray passengerAuxes = object.getJSONArray("passengerAuxes");

        List<TaobaoPassengerAux> taobaoPassengerAuxArrayList = Lists.newArrayList();


        if (passengerAuxes != null && passengerAuxes.size() != 0){
            for (Object paAuxes : passengerAuxes) {

                TaobaoPassengerAux passengerAux = objectMapper.readValue(paAuxes.toString(), TaobaoPassengerAux.class);

                taobaoPassengerAuxArrayList.add(passengerAux);
            }

        }


        TaobaoOrderRequestParam requestParam = new TaobaoOrderRequestParam();

        requestParam.setCid(cid);

        requestParam.setContact(taobaoContact);


        requestParam.setPassengers(taobaoPassengerList.toArray(new TaobaoPassenger[]{}));

        requestParam.setPassengerAuxs(taobaoPassengerAuxArrayList.toArray(new TaobaoPassengerAux[]{}));

        requestParam.setTripType(tripType);

        requestParam.setSessionId(sessionId);

        requestParam.setRouting(taobaoRouting);


        TaobaoOrderRespEntity order = orderService.order(requestParam);
        String s = JSONObject.toJSONString(order);

        log.info("---------来自 TAOBAO 的 order 返回----{}-----",s);
        String encrypt = Encrypt.encrypt(s, "Ir9ym6Bc8lCrrNvC");



        return encrypt;
    }

    public static void main(String[] args) {
        String param = "92VPxm2hGT71n9Tuuaozfk1w9aYWY1NNSKlz/LbWEA16XY6t+p7x06lojpv8Lfk6h1d12Po27oY8ThHMNed2jQ==";
        try {
            String decrypt = Encrypt.decrypt(param, "Ir9ym6Bc8lCrrNvC");

            System.out.println(decrypt);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
