package com.bjlt.spider.taobao.controller;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.entity.PlatformSourceVo;
import com.bjlt.spider.entity.RouteInfoEntity;
import com.bjlt.spider.qunaer.entity.QunaerVerifyParams;
import com.bjlt.spider.qunaer.entity.QunarRoutings;
import com.bjlt.spider.taobao.entity.TaoBaoSegment;
import com.bjlt.spider.taobao.entity.TaobaoRouting;
import com.bjlt.spider.taobao.entity.TaobaoVerifyRespEntity;
import com.bjlt.spider.taobao.service.TaobaoVerifyService;
import com.bjlt.spider.util.RedisPrefix;
import com.bjlt.spider.utils.LoadParam;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonAppend;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 20:13 2018/1/15
 */
@RestController
@RequestMapping(value = "/taobao")
@Slf4j
public class VerifyController {

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private TaobaoVerifyService verifyService;

    @PostMapping(value = "verify", consumes = MediaType.TEXT_PLAIN_VALUE)
    public TaobaoVerifyRespEntity verify(HttpServletRequest request) throws Exception {

        log.info("---------来自 TAOBAO 的 verify 请求---------");

        String param = LoadParam.read(request.getInputStream());
        JSONObject object = JSONObject.parseObject(param);
        String cid = object.getString("cid");
        String tripType = object.getString("tripType");

        String routingString = object.getJSONObject("routing").toJSONString();

        ObjectMapper objectMapper = new ObjectMapper();

        TaobaoRouting taobaoRouting = objectMapper.readValue(routingString, TaobaoRouting.class);


        QunaerVerifyParams qunaerVerifyParams = new QunaerVerifyParams();


        //拿到默认人数
        //TODO 大 BUG
        PlatformSourceVo vo = (PlatformSourceVo) redisTemplate.opsForValue().get(RedisPrefix.AIRSOURCEPLATFORM.getPrefix()+cid+"1");

        Integer defultCount = null;
        if (vo != null){
            defultCount = vo.getPassengerCount();
        }else {
            defultCount = 3;
        }

        qunaerVerifyParams.setAdultNum(defultCount);

        qunaerVerifyParams.setCid(cid);

        qunaerVerifyParams.setTripType(tripType);

        QunarRoutings qunarRoutings = new QunarRoutings();


        qunarRoutings.setData(taobaoRouting.getData());


        TaoBaoSegment[] fromSegments = taobaoRouting.getFromSegments();

        List<TaoBaoSegment> fromTaoBaoSegmentList = Arrays.stream(fromSegments).collect(Collectors.toList());


        List<RouteInfoEntity> fromEntity =  changeEntity(fromTaoBaoSegmentList);



        qunarRoutings.setFromSegments(fromEntity);


        if (tripType.equals("2")){
            TaoBaoSegment[] retSegments = taobaoRouting.getRetSegments();

            List<TaoBaoSegment> retTaoBaoSegmentList = Arrays.stream(retSegments).collect(Collectors.toList());


            List<RouteInfoEntity> retEntity = changeEntity(retTaoBaoSegmentList);

            qunarRoutings.setRetSegments(retEntity);
        }

        qunarRoutings.setTripType(tripType);

        qunaerVerifyParams.setRouting(qunarRoutings);



        return verifyService.verify(qunaerVerifyParams);
    }


    public static List<RouteInfoEntity> changeEntity(List<TaoBaoSegment> taoBaoSegments){


        List<RouteInfoEntity> routeInfoEntities = Lists.newArrayList();

        for (TaoBaoSegment taoBaoSegment : taoBaoSegments) {
            RouteInfoEntity entity = new RouteInfoEntity();

            entity.setAircraftCode(taoBaoSegment.getAircraftCode());

            entity.setCabin(taoBaoSegment.getCabin());

            entity.setCarrier(taoBaoSegment.getCarrier());

            entity.setFlightNumber(taoBaoSegment.getFlightNumber());

            entity.setDepAirport(taoBaoSegment.getDepAirport());

            entity.setDepTime(taoBaoSegment.getDepTime());

            entity.setArrAirport(taoBaoSegment.getArrAirport());


            entity.setArrTime(taoBaoSegment.getArrTime());

            entity.setCodeshare(taoBaoSegment.isCodeShare());


            routeInfoEntities.add(entity);

        }
        return routeInfoEntities;
    }
}
