package com.bjlt.spider.taobao.entity;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 16:53 2018/1/15
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class TaoBaoSegment implements Serializable{

    private static final long serialVersionUID = 1157454280591999437L;
    /**
     * 航司 IATA 二字码，必须与 flightNumber 航司相同
     */
    private String carrier;

    /**
     * 航班号，如:CA123
     */
    private String flightNumber;

    /**
     * 出发机场 IATA 三字码
     */
    private String depAirport;

    /**
     * 起飞日期时间，格式:YYYYMMDDHHMM 例: 201203100300 表示 2012 年 3 月 10 日 3 时 0 分
     */
    private String depTime;

    /**
     * 到达机场 IATA 三字码
     */
    private String arrAirport;

    /**
     * 起飞日期时间，格式:YYYYMMDDHHMM 例: 201203101305 表示 2012 年 3 月 10 日 13 时 5 分
     */
    private String arrTime;

    /**
     * 经停地，/分隔城市三字码 (非必须)
     */
    private String stopCities;

    /**
     * 代码共享标识(true 代码共享/false 非代码共享)
     */
    private boolean codeShare;

    /**
     * 实际承运航司,当 codeShare=true 时, operatingCarrier 不能为空
     */
    private String operatingCarrier;

    /**
     * 实际承运航班号
     (非必须，codeShare 为 true 时请提供)
     */
    private String operatingFlightNo;

    /**
     * 出发航站楼(非必须)，有辅营产品的必须提供。
     */
    private String departureTerminal;

    /**
     * 到达航站楼(非必须)
     */
    private String arrivingTerminal;

    /**
     * 舱位
     */
    private String cabin;

    /**
     * 机型 ,IATA 标准 3 字码,比如:737 、320 等，如果不
     确定机型请传 JET, 平台会并过滤下列非机票运价信 息，如:BUS|ICE|LCH|LMO|MTL|RFS|TGV|THS|THT|TRN|TSL|
     */
    private String aircraftCode;

    /**
     * 舱位数(必传)
     如果此节点不传默认为 9，如果传 A 视为 9，数字仅允 许传 1-9 的个位数
     */
    private String seatCount;

    /**
     * 舱位等级: Y 代表经济舱,C 代表商务舱, F 代表头等 舱。
     */
    private String cabinClass;

    /**
     * 按航段提供 farebasis，若返回含有多个航段，需要每 个航段都返回一个 farebasis，多个之间用英文逗号隔 开
     */
    private String fareBasis;

    /**
     * 表示运价类型，指代提前销售折扣运价等不同运价类
     型;
     fareTypeCode 为字符串格式，不允许填写特殊字符
     每个航段必须对应 1 个，若多个航段运价所属同一个 faretypecode，则相同 code 填多次
     */
    private String fareTypeCode;

    /**
     * 四位字符，数字或英文字符组合，不支持特殊字符. 若多个航段使用同一个运价，则相同 ruleNo 填多次
     */
    private String ruleNo;

    /**
     * 托运行李额说明
     */
    private TaobaoBaggage baggage;
}
