package com.bjlt.spider.taobao.entity;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 16:33 2018/1/15
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class TaobaoRule implements Serializable{

    private static final long serialVersionUID = -2855252244380440069L;
    /**
     * 是否允许退票:0 表示不允许,1 表示允许;
     */
    private Integer hasRefund;

    /**
     * 退票费使用 3 位字母货币单位，比如:”CNY”, “USD”,”EUR”,”IDR”。若没有改字段或者为空 则默认为“CNY”;代表人民币;
     */
    private String refundCurrency;

    /**
     * 退票规定，最大 300 个字符; 可输入格式如:200-72-
     300-48-1000-0-*，表示 72 小时前退票手续费 200; 48 小时到 72 小时，退票手续费 300;飞机起飞不足 48 小时;退票手续费 1000;飞机起飞后不予改期(输 入*);
     或
     50%-0-*，表示飞机起飞不足 48 小时;退票手续费为 票面价的 50%;飞机起飞后不予退票(输入*);
     2017.3 月要求上传退改签格式化:如果 hasRefund=1,则本字段必须为格式化(数字横线和星 号表示)，不能再使用之前的自由文本或者空，否则会 因为无法解析不能展示。
     */
    private String refund;

    /**
     * 部分未使用可否退票,0 表示不允许,1 表示允许;单程直 飞为 0 ;
     */
    private Integer partRefund;


    /**
     * 部分退票费使用 3 位字母货币单位，比如:”CNY”, “USD”,”EUR”,”IDR”。若没有改字段或者为空 则默认为“CNY”;代表人民币;
     */
    private String partRefundCurrency;

    /**
     * 部分未使用退票费用,见“refund”格式说明. partRefund=1 时必填
     */
    private String partRefundPrice;

    /**
     * 部分未使用退票费用按每个航段收还是全程收: "ALL" 表示按全程收(默认)，"EACH"表示按每个航段收;
     */
    private String partRefundSeg;

    /**
     * 是否允许改期:0 表示不允许,1 表示允许
     */
    private Integer hasEndorse;

    /**
     * 使用 3 位字母货币单位，比如:”CNY”, “USD”,”
     EUR”,”IDR”。若没有改字段或者为空则默认为 “CNY”;代表人民币;
     */
    private String endorseCurrency;

    /**
     * 改签规定，最大 300 个字符 ;可输入格式如:200-72- 300-48-1000-0-*,代表 72 小时前 改期手续费 200 ;48 小时到 72 小时之间,改期手续费 300 ; 飞机起飞不足 48 小时,手续费 1000;起飞后不予改期(输入*),为空表示 按航司规定 ;
     2017.3 月要求上传退改签格式化:如果 hasRefund=1,则本字段必须为格式化(数字横线和星 号表示)，不能再使用之前的自由文本或者空，否则会 因为无法解析不能展示。
     */
    private String endorse;

    /**
     * 部分改期规则,0 表示不允许,1 表示允许;单程直飞为 0 ，当往返时此处表示为返回是否允许改期;
     */
    private Integer partEndorse;

    /**
     * 3 位字母货币单位，比如:”CNY”, “USD”,” EUR”,”IDR”。若没有改字段或者为空则默认为 “CNY”;代表人民币;
     */
    private String partEndorseCurrency;

    /**
     * 部分(回程)改期费用,见“refund”格式说明.
     partEndorse=1 时必填(单位以所输入的币种为准, 如:USD, EUR, 或 IDR 等)
     */
    private String partEndorsePrice;

    /**
     * 签转规则,0 表示不支持,1 表示支持 。
     目前不支持签转。默认为 0 不支持。
     */
    private Integer endorsement;

    /**
     * 是否提供免费行李额:0 表示不提供,1 表示提供
     */
    private Integer hasBaggage;

    /**
     * 行李额规定，最大 300 个字符
     */
    private String baggage;

    /**
     * 是否有 noshow 规则,0 表示没有规则限制,1 表示有规 则限制
     */
    private Integer hasNoShow;

    /**
     * Noshow 时限,输入正整数，单位是小时;
     */
    private Integer noShowLimitTime;

    /**
     * 3 位字母货币单位，比如:”CNY”, “USD”,” EUR”,”IDR”。若没有改字段或者为空则默认为 “CNY”;代表人民币;
     */
    private String noShowCurrency;

    /**
     * Noshow 罚金,输入正整数或者百分比
     */
    private String penalty;

    /**
     * Noshow 特殊规则:0(无) 1(Noshow 时限内不允许退 票) 2(Noshow 时限内不允许改期) 3(Noshow 时限内 不允许退票和改期),
     */
    private String specialNoShow;

    /**
     * 其他说明，最大 300 个字符
     */
    private String other;
}
