package com.bjlt.spider.taobao.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.common.PlatformCids;
import com.bjlt.spider.commonhandler.AirSoucePolicyForOrderHandler;
import com.bjlt.spider.entity.*;
import com.bjlt.spider.interfaces.ControRpc;
import com.bjlt.spider.mq.BaseMqSender;
import com.bjlt.spider.orderInfo.entity.LocalAdditionalServices;
import com.bjlt.spider.orderInfo.service.LocalAdditionalServicesService;
import com.bjlt.spider.orderInfo.service.OrOrdertbService;
import com.bjlt.spider.qunaer.entity.ContactInfo;
import com.bjlt.spider.qunaer.entity.PassengersInfo;
import com.bjlt.spider.qunaer.entity.QunaerOrderParams;
import com.bjlt.spider.qunaer.entity.QunarRoutings;
import com.bjlt.spider.taobao.entity.*;
import com.bjlt.spider.taobao.service.BaseService;
import com.bjlt.spider.taobao.service.TaobaoOrderService;
import com.bjlt.spider.util.RespEntity;
import com.bjlt.spider.util.UUIDUtils;
import com.bjlt.spider.utils.ActionIdUtils;
import com.bjlt.spider.utils.OrderUtils;
import com.bjlt.spider.utils.PnrUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.math.BigDecimal;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import static com.bjlt.spider.taobao.controller.VerifyController.changeEntity;
import static com.bjlt.spider.taobao.service.impl.TaobaoSearchServiceImpl.changeRoutingFromCommon;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 15:13 2018/1/16
 */
@Service
@Slf4j
public class TaobaoOrderServiceImpl implements TaobaoOrderService {


    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private ControRpc controRpc;

    @Autowired
    private BaseMqSender mqSender;

    @Autowired
    private AirSoucePolicyForOrderHandler orderHandler;


    @Autowired
    private LocalAdditionalServicesService additionalServicesService;

    @Autowired
    private OrOrdertbService orOrdertbService;

    private ObjectMapper objectMapper = new ObjectMapper();

    @Override
    public TaobaoOrderRespEntity order(TaobaoOrderRequestParam param) {


        TaobaoOrderRespEntity entity = new TaobaoOrderRespEntity();

        //还是得先去搜索
        List<SearchResponseEntity> responseEntityArrayList = Lists.newArrayList();
        TaobaoVerifyRespEntity verifyRespEntity = new TaobaoVerifyRespEntity();

        //通过data 解析出航班信息
        String data = param.getRouting().getData();


        Integer childNum = new Long(Arrays.stream(param.getPassengers()).filter(t -> t.getAgeType() == 1).count()).intValue();


        QunarRoutings qunarRoutings = new QunarRoutings();

        qunarRoutings.setTripType(param.getTripType());


        List<TaoBaoSegment> fromTaoBaoSegmentList = Arrays.stream(param.getRouting().getFromSegments()).collect(Collectors.toList());

        List<RouteInfoEntity> fromEntity = changeEntity(fromTaoBaoSegmentList);


        qunarRoutings.setFromSegments(fromEntity);


        if (param.getTripType().equals("2")) {
            TaoBaoSegment[] retSegments = param.getRouting().getRetSegments();

            List<TaoBaoSegment> retTaoBaoSegmentList = Arrays.stream(retSegments).collect(Collectors.toList());


            List<RouteInfoEntity> retEntity = changeEntity(retTaoBaoSegmentList);

            qunarRoutings.setRetSegments(retEntity);
        }


        Map<String, Object> ret = BaseService.getRet(data, qunarRoutings, param.getTripType(), childNum, param.getPassengers().length);


        SearchRequestAfterOta searchRequestAfterOta = (SearchRequestAfterOta) ret.get("otaConfig");

        QunarRoutings routings = (QunarRoutings) ret.get("routings");

        String pid = (String) ret.get("pid");

        RespEntity spiderEntity = null;
        //现在去搜索
        try {
            spiderEntity = controRpc.createSpider(searchRequestAfterOta);
        } catch (SQLException e) {
            e.printStackTrace();
        }


        if (spiderEntity.getStatus() != 200) {

            entity.setMsg("ERROR");
            entity.setStatus(999);

            return entity;
        }


        Object attachment = spiderEntity.getAttachment();

        List retList = (List) attachment;

        if (retList.size() == 0) {
            entity.setMsg("ERROR");
            entity.setStatus(999);
            return entity;
        }
        log.info("---------拿到搜索服务的结果---------");

        retList.forEach(t -> {

            String searchEntityString = JSONObject.toJSONString(t);


            try {
                SearchResponseEntity responseEntity = objectMapper.readValue(searchEntityString, SearchResponseEntity.class);
                if (responseEntity.getUuid().equals(searchRequestAfterOta.getUuid())) {
                    responseEntityArrayList.add(responseEntity);
                }


            } catch (IOException e) {
                e.printStackTrace();
            }

        });
        List<QunarRoutings> qunarRoutings1 = orderHandler.handlerList(responseEntityArrayList, param.getCid(), routings, param.getTripType());

        if (qunarRoutings1 == null || qunarRoutings1.size() == 0) {
            entity.setMsg("ERROR");
            entity.setStatus(999);
            return entity;
        }

        param.setSessionId(UUIDUtils.getUUID());
        qunarRoutings1.get(0).setData(param.getRouting().getData());
        QunarRoutings routings1 = qunarRoutings1.get(0);

        String pnr = PnrUtils.getPnr();

        entity.setPnrCode(pnr);

        BigDecimal allPrice = new BigDecimal(param.getPassengers().length).multiply(qunarRoutings1.get(0).getAdultPrice());

        //查看需要下单的人数够不够

        //总人数是
        Integer psCount = param.getPassengers().length;

        //现在有座位个数
        Integer maxSeats = routings1.getMaxSeats();

        if (psCount > maxSeats) {
            entity.setMsg("座位数不够");
            entity.setStatus(2);
            return entity;
        }


        String name = "";
        String bir = "";
        String gender = "";
        String card = "";


        List<PassengersInfo> passengersInfoList = new ArrayList<>();

        if (param.getPassengers().length != 0) {
            for (TaobaoPassenger taobaoPassenger : param.getPassengers()) {
                PassengersInfo passengersInfo = new PassengersInfo();
                passengersInfo.setBirthday(taobaoPassenger.getBirthday());
                passengersInfo.setCardType(taobaoPassenger.getCardType());
                passengersInfo.setCardNum(taobaoPassenger.getCardNum());
                passengersInfo.setNationality(taobaoPassenger.getNationality());
                passengersInfo.setAgeType(taobaoPassenger.getAgeType());
                passengersInfo.setName(taobaoPassenger.getName());
                passengersInfo.setGender(taobaoPassenger.getGender());
                passengersInfoList.add(passengersInfo);

            }
        }


        String orderId = ActionIdUtils.getUUID();
        OrOrdertb orOrdertb = OrOrdertb.builder()
                .ordersource(PlatformCids.TAOBAO)
                .orderno(orderId)
                .flighttype(param.getTripType())
                .ordercodestatus(OrderCodeStatus.ORDER_INIT.getStatus())
                .orderstatus(OrderStatus.ORDER_INIT.getStatus())
                .airlinecount(routings1.getFromSegments().size())
                .childrencount((int) Arrays.stream(param.getPassengers()).filter(t -> t.getAgeType() == 1).count())
                .adultscount((int) Arrays.stream(param.getPassengers()).filter(t -> t.getAgeType() == 0).count())
                .passengercount(param.getPassengers().length)
                .adultprice(routings1.getAdultPrice())
                .adulttax(routings1.getAdultTax())
                .childtax(routings1.getChildTax())
                .childprice(routings1.getChildPrice())
                .passengerName(name)
                .passengerBirth(bir)
                .policyid(pid)
                .passengerCardnum(card)
                .passengerGender(gender)
                .pnrcode(pnr)
                .receiver(param.getContact().getName())
                .postcode(param.getContact().getPostcode())
                .address(param.getContact().getAddress())
                .createtime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()))
                .lastupdated(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()))
                .operator("System")
                .build();
        orOrdertbService.insertSelective(orOrdertb);


        List<LocalAdditionalServices> localAdditionalServicesList = new ArrayList<>();


        TaobaoPassengerAux[] passengerAuxs = param.getPassengerAuxs();


        Arrays.stream(passengerAuxs).forEach(t -> {
            LocalAdditionalServices services = new LocalAdditionalServices();
            services.setCounts(t.getBookNum());
            services.setPassenger(t.getName());
            services.setPrice((double) t.getProductItem().getOnlinePrice());
            services.setFlightNumber(t.getSegment().getFlightNumber());

            services.setFlightSegment(t.getSegment().toString());
            services.setOrderNo(orderId);

            services.setProductType(String.valueOf(t.getProductItem().getProductType()));

            services.setEditUser("System");

            localAdditionalServicesList.add(services);

        });


        for (LocalAdditionalServices services : localAdditionalServicesList) {
            additionalServicesService.insertSelective(services);
        }

        mqSender.send(param, "_order");

        entity.setStatus(0);
        entity.setMsg("OK");
        entity.setSessionId(UUIDUtils.getUUID());

        entity.setOrderNo(orderId);
        entity.setPnrCode(pnr);

        entity.setMaxSeats(routings1.getMaxSeats());

        entity.setBookingGds("AK");


        List<TaobaoRouting> taobaoRoutingList = changeRoutingFromCommon(qunarRoutings1, param.getTripType());


        entity.setRouting(taobaoRoutingList.get(0));

        QunaerOrderParams qunaerOrderParams = new QunaerOrderParams();

        qunaerOrderParams.setTripType(param.getTripType());

        qunaerOrderParams.setSessionId(entity.getSessionId());

        qunaerOrderParams.setOrderSource(PlatformCids.TAOBAO);

        qunaerOrderParams.setPassengers(passengersInfoList.toArray(new PassengersInfo[]{}));

        qunaerOrderParams.setOrderID(orderId);

        qunaerOrderParams.setRouting(qunarRoutings1.get(0));

        try {
            new Thread(new OrderUtils(redisTemplate, orOrdertbService, qunaerOrderParams), "ORDER").start();
        } catch (Exception e) {
            log.info("自动生单失败");
            return entity;
        }

        return entity;
    }


}
