package com.bjlt.spider.taobao.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.common.MethedNames;
import com.bjlt.spider.commonhandler.AirSoucePolicyForSearchHandler;
import com.bjlt.spider.entity.RouteInfoEntity;
import com.bjlt.spider.entity.SearchRequestAfterOta;
import com.bjlt.spider.entity.SearchResponseEntity;
import com.bjlt.spider.interfaces.ControRpc;
import com.bjlt.spider.qunaer.entity.QunaerSearchParams;
import com.bjlt.spider.qunaer.entity.QunarRoutings;
import com.bjlt.spider.taobao.entity.TaoBaoSegment;
import com.bjlt.spider.taobao.entity.TaobaoRouting;
import com.bjlt.spider.taobao.entity.TaobaoRule;
import com.bjlt.spider.taobao.entity.TaobaoSearchRespEntity;
import com.bjlt.spider.taobao.service.TaobaoSearchService;
import com.bjlt.spider.util.RespEntity;
import com.bjlt.spider.utils.OtaConfigGenerateKey;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 17:23 2018/1/15
 */
@Slf4j
@Service
public class TaobaoSearchServiceImpl implements TaobaoSearchService {

    @Autowired
    private ControRpc controRpc;

    private ObjectMapper objectMapper = new ObjectMapper();

    @Autowired
    private AirSoucePolicyForSearchHandler searchHandler;


    @Override
    public TaobaoSearchRespEntity search(QunaerSearchParams params) {

        TaobaoSearchRespEntity respEntity = new TaobaoSearchRespEntity();

        List<QunarRoutings> routingsList = Lists.newArrayList();
        List<SearchResponseEntity> responseEntityArrayList = Lists.newArrayList();


        //获取数据结果
        SearchRequestAfterOta ota = SearchRequestAfterOta.builder()
                .adultNum(params.getAdultNum())
                .childNum(params.getChildNum())
                .fromCity(params.getFromCity())
                .toCity(params.getToCity())
                .retDate(params.getRetDate())
                .fromDate(params.getFromDate())
                .cid(params.getCid())
                .methodName(MethedNames.search)
                .tripType(params.getTripType())
                .cache(true)
                .build();


        String uuid = "";
        if (params.getTripType().equals("1")) {
            uuid = OtaConfigGenerateKey.fromKey(ota);
        } else {
            uuid = OtaConfigGenerateKey.allKey(ota);
        }

        ota.setUuid(uuid);

        if ("5".equals(params.getTripType())) {
            respEntity.setMsg("不支持多程");
            respEntity.setStatus(999);
            return respEntity;
        }


        try {
            log.info("开始向控制中心发送search请求{}", params.toString());
            RespEntity spiderEntity = controRpc.createSpider(ota);


            if (spiderEntity.getStatus() != 200) {
                respEntity.setMsg("SERACH FAIL");
                respEntity.setStatus(999);
                return respEntity;
            }


            Object attachment = spiderEntity.getAttachment();

            List retList = (List) attachment;


            retList.forEach(t -> {

                String searchEntityString = JSONObject.toJSONString(t);


                try {
                    SearchResponseEntity responseEntity = objectMapper.readValue(searchEntityString, SearchResponseEntity.class);

                    if (responseEntity.getUuid().equals(ota.getUuid())) {
                        responseEntityArrayList.add(responseEntity);
                    }


                } catch (IOException e) {
                    e.printStackTrace();
                }

            });

            if (retList.size() == 0) {
                respEntity.setMsg("SERACH FAIL");
                respEntity.setStatus(999);
                return respEntity;
            }
            // log.info("---------拿到搜索服务的结果{}---------", responseEntityArrayList);


            routingsList = searchHandler.handlerList(responseEntityArrayList, params.getCid(), null, params.getTripType());


            routingsList.stream().forEach(t -> t.setCurreny("CNY"));
            routingsList.stream().forEach(t -> t.setFromCity(params.getFromCity()));
            routingsList.stream().forEach(t -> t.setToCity(params.getToCity()));


            List<QunarRoutings> finalroutingsList = new ArrayList<>();

            if (params.getTripType().equals("2")) {
                Map<Map<List<RouteInfoEntity>, List<RouteInfoEntity>>, QunarRoutings> map = new HashMap<>();

                for (QunarRoutings qunarRoutings : routingsList) {
                    List<RouteInfoEntity> fromSegments = qunarRoutings.getFromSegments();
                    List<RouteInfoEntity> retSegments = qunarRoutings.getRetSegments();
                    Map<List<RouteInfoEntity>, List<RouteInfoEntity>> listMap = new HashMap<>();
                    listMap.put(fromSegments, retSegments);
                    QunarRoutings qunarRoutings1 = map.get(listMap);
                    if (qunarRoutings1 == null || !qunarRoutings.getTag().equals(qunarRoutings1.getTag())) {
                        map.put(listMap, qunarRoutings);
                    } else if (qunarRoutings1.getAdultPrice().compareTo(qunarRoutings.getAdultPrice()) > 0) {
                        map.remove(listMap);
                        map.put(listMap, qunarRoutings);
                    }
                }
                finalroutingsList = new ArrayList<>(map.values());
            } else {
                finalroutingsList = routingsList;
            }

            List<TaobaoRouting> taobaoRoutingList = changeRoutingFromCommon(finalroutingsList, params.getTripType());


            List<TaoBaoSegment> segments = new ArrayList<>();
            if (params.getTripType().equals("1")) {

                taobaoRoutingList.stream().forEach(t -> t.setRetSegments(segments.toArray(new TaoBaoSegment[]{})));
            }

            respEntity.setRoutings(taobaoRoutingList.toArray(new TaobaoRouting[]{}));
            respEntity.setMsg("OK");
            respEntity.setStatus(0);
            return respEntity;

        } catch (Exception e) {
            e.printStackTrace();
            respEntity.setMsg("SERACH FAIL");
            respEntity.setStatus(999);
            return respEntity;
        }

    }


    public static List<TaobaoRouting> changeRoutingFromCommon(List<QunarRoutings> routingsList, String tripType) {

        List<TaobaoRouting> taobaoRoutingList = Lists.newArrayList();

        for (QunarRoutings qunarRoutings : routingsList) {

            TaobaoRouting taobaoRouting = new TaobaoRouting();

            taobaoRouting.setData(qunarRoutings.getData());

            taobaoRouting.setAdultPrice(qunarRoutings.getAdultPrice().intValue());

            taobaoRouting.setAdultTax(qunarRoutings.getAdultTax().intValue());

            taobaoRouting.setChildPrice(qunarRoutings.getChildPrice().intValue());


            taobaoRouting.setChildTax(qunarRoutings.getChildTax().intValue());


            taobaoRouting.setMinPassengerCount(1);

            taobaoRouting.setMaxPassengerCount(qunarRoutings.getMaxSeats());

            taobaoRouting.setGvChildRule(1);


            taobaoRouting.setAdultTaxType(0);

            taobaoRouting.setChildTaxType(0);

            taobaoRouting.setApplyType(0);

            taobaoRouting.setPriceType(0);

            taobaoRouting.setNationality("");

            taobaoRouting.setNationalityType(0);


            if (qunarRoutings.getRule() != null) {
                TaobaoRule taobaoRule = new TaobaoRule();
                BeanUtils.copyProperties(qunarRoutings.getRule(), taobaoRule);
                taobaoRouting.setRule(taobaoRule);
            }


            List<RouteInfoEntity> fromSegments = qunarRoutings.getFromSegments();

            List<TaoBaoSegment> taoBaoSegmentList = Lists.newArrayList();

            changeSegment(fromSegments, taoBaoSegmentList, qunarRoutings);


            taobaoRouting.setFromSegments(taoBaoSegmentList.toArray(new TaoBaoSegment[]{}));


            if (tripType.equals("2")) {
                List<RouteInfoEntity> retSegments = qunarRoutings.getRetSegments();

                List<TaoBaoSegment> taoBaoretSegmentList = Lists.newArrayList();

                changeSegment(retSegments, taoBaoretSegmentList, qunarRoutings);

                taobaoRouting.setRetSegments(taoBaoretSegmentList.toArray(new TaoBaoSegment[]{}));
            }

            taobaoRoutingList.add(taobaoRouting);

        }

        return taobaoRoutingList;

    }


    public static void changeSegment(List<RouteInfoEntity> routeInfoEntities, List<TaoBaoSegment> taoBaoSegments, QunarRoutings qunarRoutings) {

        for (RouteInfoEntity segment : routeInfoEntities) {
            TaoBaoSegment taoBaoSegment = new TaoBaoSegment();

            taoBaoSegment.setCarrier(segment.getCarrier());

            taoBaoSegment.setAircraftCode(segment.getAircraftCode());

            taoBaoSegment.setFlightNumber(segment.getFlightNumber());

            taoBaoSegment.setDepAirport(segment.getDepAirport());

            taoBaoSegment.setDepTime(segment.getDepTime());

            taoBaoSegment.setArrAirport(segment.getArrAirport());

            taoBaoSegment.setArrTime(segment.getArrTime());

            taoBaoSegment.setStopCities(segment.getStopCities() == null ? null : segment.getStopCities());

            taoBaoSegment.setCodeShare(segment.isCodeshare());

            if (segment.getCabinClass() != null) {
                taoBaoSegment.setCabin(segment.getCabin());

                taoBaoSegment.setCabinClass(segment.getCabinClass());
            } else {
                taoBaoSegment.setCabin(segment.getCabin());

                taoBaoSegment.setCabinClass(segment.getCabinClass());
            }


            taoBaoSegment.setSeatCount(qunarRoutings.getMaxSeats().toString());
            taoBaoSegments.add(taoBaoSegment);
        }


    }

}