package com.bjlt.spider.taobao.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.commonhandler.AirSoucePolicyForVerifyHandler;
import com.bjlt.spider.entity.SearchRequestAfterOta;
import com.bjlt.spider.entity.SearchResponseEntity;
import com.bjlt.spider.interfaces.ControRpc;
import com.bjlt.spider.qunaer.entity.QunaerVerifyParams;
import com.bjlt.spider.qunaer.entity.QunarRoutings;
import com.bjlt.spider.taobao.entity.TaoBaoSegment;
import com.bjlt.spider.taobao.entity.TaobaoRouting;
import com.bjlt.spider.taobao.entity.TaobaoRule;
import com.bjlt.spider.taobao.entity.TaobaoVerifyRespEntity;
import com.bjlt.spider.taobao.service.BaseService;
import com.bjlt.spider.taobao.service.TaobaoSearchService;
import com.bjlt.spider.taobao.service.TaobaoVerifyService;
import com.bjlt.spider.util.RespEntity;
import com.bjlt.spider.util.UUIDUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static com.bjlt.spider.taobao.service.impl.TaobaoSearchServiceImpl.changeRoutingFromCommon;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 21:08 2018/1/15
 */
@Service
@Slf4j
public class TaobaoVerifyServiceImpl implements TaobaoVerifyService {


    @Autowired
    private ControRpc controRpc;

    @Autowired
    private AirSoucePolicyForVerifyHandler verifyHandler;


    private ObjectMapper objectMapper = new ObjectMapper();


    @Override
    public TaobaoVerifyRespEntity verify(QunaerVerifyParams verifyParams) {

        List<SearchResponseEntity> responseEntityArrayList = Lists.newArrayList();
        TaobaoVerifyRespEntity verifyRespEntity = new TaobaoVerifyRespEntity();

        //通过data 解析出航班信息
        String data = verifyParams.getRouting().getData();



        Map<String, Object> ret = BaseService.getRet(data, verifyParams.getRouting(), verifyParams.getTripType(), verifyParams.getChildNum(),verifyParams.getAdultNum());


       SearchRequestAfterOta searchRequestAfterOta = (SearchRequestAfterOta) ret.get("otaConfig");

        QunarRoutings routings = (QunarRoutings) ret.get("routings");

        RespEntity spiderEntity = null;
        //现在去搜索
        try {
            spiderEntity = controRpc.createSpider(searchRequestAfterOta);
        } catch (SQLException e) {
            e.printStackTrace();
        }



        if (spiderEntity.getStatus() != 200) {

            verifyRespEntity.setMsg("FAIL");
            verifyRespEntity.setStatus(999);

            return verifyRespEntity;
        }


        Object attachment = spiderEntity.getAttachment();

        List retList = (List) attachment;

        if (retList.size() == 0) {
            verifyRespEntity.setMsg("FAIL");
            verifyRespEntity.setStatus(999);
            return verifyRespEntity;
        }
        log.info("---------拿到搜索服务的结果---------");

        retList.forEach(t -> {

            String searchEntityString = JSONObject.toJSONString(t);


            try {
                SearchResponseEntity responseEntity = objectMapper.readValue(searchEntityString, SearchResponseEntity.class);
                if (responseEntity.getUuid().equals(searchRequestAfterOta.getUuid())){
                    responseEntityArrayList.add(responseEntity);

                }

            } catch (IOException e) {
                e.printStackTrace();
            }

        });
        List<QunarRoutings> qunarRoutings = verifyHandler.handlerList(responseEntityArrayList, verifyParams.getCid(), routings, verifyParams.getTripType());

        if (qunarRoutings == null || qunarRoutings.size() == 0) {
            verifyRespEntity.setMsg("FAIL");
            verifyRespEntity.setStatus(999);
            return verifyRespEntity;
        }

        verifyRespEntity.setSessionId(UUIDUtils.getUUID());


        List<TaobaoRouting> taobaoRoutingList = changeRoutingFromCommon(qunarRoutings, verifyParams.getTripType());


        TaobaoRouting taobaoRouting = taobaoRoutingList.get(0);

        if (taobaoRouting.getRetSegments() == null){
            List<TaoBaoSegment> segments = new ArrayList<>();
            taobaoRouting.setRetSegments(segments.toArray(new TaoBaoSegment[]{}));
        }


        TaobaoRule rule = taobaoRouting.getRule();
        verifyRespEntity.setRule(rule);

        taobaoRouting.setRule(null);

        taobaoRouting.setData(data);

        verifyRespEntity.setRouting(taobaoRouting);



        verifyRespEntity.setStatus(0);

        verifyRespEntity.setMaxSeats(taobaoRouting.getMaxPassengerCount());

        return verifyRespEntity;

    }

}

