package com.bjlt.spider.utils;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.common.DataEntity;
import com.bjlt.spider.entity.*;
import com.bjlt.spider.orderInfo.service.OrOrdertbService;
import com.bjlt.spider.qunaer.entity.PassengersInfo;
import com.bjlt.spider.qunaer.entity.QunaerOrderParams;
import com.bjlt.spider.qunaer.entity.QunarRoutings;
import com.bjlt.spider.taobao.entity.TaobaoPassengerAux;
import com.bjlt.spider.util.RedisPrefix;
import com.bjlt.spider.util.RespEntity;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.RedisTemplate;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author: Scott
 * @Description:
 * @Date: Create in 15:18 2018/2/6
 */
@Slf4j
public class OrderUtils implements Runnable {
    private RedisTemplate<String, Object> redisTemplate;

    private OrOrdertbService orOrdertbService;

    private QunaerOrderParams params;

    public OrderUtils(RedisTemplate<String, Object> redisTemplate, OrOrdertbService orOrdertbService, QunaerOrderParams params) {
        this.redisTemplate = redisTemplate;
        this.orOrdertbService = orOrdertbService;
        this.params = params;
    }

    private ObjectMapper objectMapper = new ObjectMapper();

    @Override
    public void run() {
        OrderRequestEntity entity = generateEntity(params);

        RespEntity order = null;

        RespEntity retOrder = null;


        OrOrdertb orOrdertb = null;

        QunarRoutings routing = params.getRouting();
        //首先生去程的单
        List<RouteInfoEntity> fromSegments = routing.getFromSegments();
        String carrier = fromSegments.get(0).getCarrier();

        log.info("carrier---------{}", carrier);

        String sourceName = getSourceName(carrier);

        if (sourceName == null) {
            log.info("---------手工生单---------");
            return;
        }

        log.info("sourceName---------{}", sourceName);
        Integer sourceId = SourceEnum.valueOf(sourceName).getSourceId();
        Set<Object> members = redisTemplate.opsForSet().members(RedisPrefix.AIRSOURCEMUNE.getPrefix() + sourceId);

        AirSourceMenuVo vo = null;
        for (Object member : members) {
            String s = JSONObject.toJSONString(member);
            AirSourceMenuVo airSourceMenuVo = null;
            try {
                airSourceMenuVo = objectMapper.readValue(s, AirSourceMenuVo.class);
            } catch (IOException e) {
                e.printStackTrace();
            }
            if (airSourceMenuVo.getIsDefault().equals(1)) {
                vo = airSourceMenuVo;
            }
        }
        String orderUrl = vo.getOrderUrl();

        log.info("下单地址是 {}", orderUrl);

        String retOrderUrl = "";
        String retSourceName = "";

        if (params.getTripType().equals("2")) {
            //现在返程的单
            List<RouteInfoEntity> retSegments = routing.getRetSegments();
            String retCarrier = retSegments.get(0).getCarrier();
            retSourceName = getSourceName(retCarrier);

            Integer retSourceId = SourceEnum.valueOf(retSourceName).getSourceId();
            Set<Object> retMembers = redisTemplate.opsForSet().members(RedisPrefix.AIRSOURCEMUNE.getPrefix() + retSourceId);

            AirSourceMenuVo retVo = null;
            for (Object member : retMembers) {
                String s = JSONObject.toJSONString(member);
                AirSourceMenuVo airSourceMenuVo = null;
                try {
                    airSourceMenuVo = objectMapper.readValue(s, AirSourceMenuVo.class);
                } catch (IOException e) {
                    e.printStackTrace();
                }
                if (airSourceMenuVo.getIsDefault().equals(1)) {
                    retVo = airSourceMenuVo;
                }
            }
            retOrderUrl = retVo.getOrderUrl();

        }
        //第一种情况 都是同一种航司 不管是单程还是往返 那就一起生单


        if (retSourceName.equals(sourceName) || params.getTripType().equals("1")) {


            entity.setAccount(getAcount(sourceName));

            //存入缓存
            redisTemplate.opsForValue().set("NOTOKORDER" + params.getOrderID() + "&" + orderUrl, entity, 1, TimeUnit.DAYS);
            RespEntity orderResp = RestTemplateUtils.getOrderResp(entity, orderUrl);

            if (orderResp.getStatus() == 200) {

                List<OrOrdertb> orOrdertbs = orOrdertbService.listOrOrdertb(OrOrdertb.builder().orderno(params.getOrderID()).build());
                orOrdertb = orOrdertbs.get(0);

                orOrdertb.setOrdercodestatus(OrderCodeStatus.ORDER_INSPIDER.getStatus());
            } else {
                orOrdertb.setOrdercodestatus(OrderCodeStatus.ORDER_SPIDERERROR.getStatus());
            }


            orOrdertbService.updateByPrimaryKeySelective(orOrdertb);

        } else {
            //手工生单
            log.info("人肉生单");
        }


    }

    public OrderRequestEntity generateEntity(QunaerOrderParams params) {

        OrderRequestEntity entity = new OrderRequestEntity();

        entity.setAdultNum((int) Arrays.stream(params.getPassengers()).filter(t -> t.getAgeType() == 0).count());


        entity.setTripType(Integer.parseInt(params.getTripType()));

        String fromDate = params.getRouting().getFromDate();

        entity.setFromDate(fromDate);

        String retDate = params.getRouting().getRetDate();

        if (retDate != null && !retDate.equals("")) {
            entity.setRetDate(retDate);
        }
        entity.setFormCity(params.getRouting().getFromCity());


        entity.setToCity(params.getRouting().getToCity());


        entity.setChildNum((int) Arrays.stream(params.getPassengers()).filter(t -> t.getAgeType() == 1).count());


        entity.setFromSegments(params.getRouting().getFromSegments());

        entity.setRetSegments(params.getRouting().getRetSegments());


        entity.setContactEntity(getContact());


        BigDecimal price = params.getRouting().getPriceMap().getAdultPrice();

        BigDecimal tax = params.getRouting().getPriceMap().getAdultTax();

        entity.setFromPrice(price.add(tax));

        entity.setPassengersEntities(getPassenger(params));

        entity.setCid(params.getOrderID());


        if (params.getPassengerAuxs() != null && params.getPassengerAuxs().length != 0) {
            entity.setTaobaoPassengerAux(getaux(params));
        }


        Map<String, Object> map = new HashMap<>();
        map.put("accountCode", "1001");
        entity.setData(map);


        return entity;
    }

    //
//
    public AccountEntity getAcount(String sourceName) {
        AccountEntity accountEntity = new AccountEntity();
        if (sourceName.contains("AK")) {
            accountEntity.setUserName("CNTRIPTOPK_ADMIN");

            accountEntity.setPassWord("Tripto123");
        } else {
            accountEntity.setUserName("XIAOFAN");

            accountEntity.setPassWord("Tripto@306");
        }
        return accountEntity;

    }


    /**
     * 联系人信息
     *
     * @param
     * @return
     */
    public ContactEntity getContact() {

        ContactEntity contactEntity = new ContactEntity();
        contactEntity.setGender("M");


        contactEntity.setLastName("zhan");
        contactEntity.setFirstName("xiufan");


        contactEntity.setEmail("laopengxg@sina.com");
        contactEntity.setMobile("13691541121");

        return contactEntity;
    }


    public List<PassengersEntity> getPassenger(QunaerOrderParams params) {
        List<PassengersEntity> passengersEntities = new ArrayList<>();
        for (PassengersInfo passengersInfo : params.getPassengers()) {
            PassengersEntity passengersEntity1 = new PassengersEntity();
            BeanUtils.copyProperties(passengersInfo, passengersEntity1);
            String birthday = passengersInfo.getBirthday();
            passengersEntity1.setBirthday(birthday.substring(0, 4) + "-" + birthday.substring(4, 6) + "-" + birthday.substring(6, 8));


            String name = passengersInfo.getName();
            log.info("姓名是 {} ", name);

            passengersEntity1.setFirstName(passengersInfo.getName().split("/")[1]);
            passengersEntity1.setLastName(passengersInfo.getName().split("/")[0]);
            passengersEntities.add(passengersEntity1);
        }
        return passengersEntities;


    }

    //
    public List<com.bjlt.spider.entity.checkin.TaobaoPassengerAux> getaux(QunaerOrderParams params) {
        List<TaobaoPassengerAux> taobaoPassengerAuxes = Arrays.asList(params.getPassengerAuxs());


        List<com.bjlt.spider.entity.checkin.TaobaoPassengerAux> aus = new ArrayList<>();
        for (TaobaoPassengerAux taobaoPassengerAux : taobaoPassengerAuxes) {

            com.bjlt.spider.entity.checkin.TaobaoPassengerAux aux = new com.bjlt.spider.entity.checkin.TaobaoPassengerAux();

            BeanUtils.copyProperties(taobaoPassengerAux, aux);

            aus.add(aux);
        }


        return aus;
    }

    //
    public String getSourceName(String carrai) {


        if (carrai.equals("AK")
                || carrai.equals("FD")
                || carrai.equals("QZ")
                || carrai.equals("Z2")
                || carrai.equals("I5")
                || carrai.equals("D7")
                || carrai.equals("XJ")
                || carrai.equals("XT")
                || carrai.equals("DJ")) {
            return "AK_WEB";
        } else if (carrai.equals("SL") || carrai.equals("JT") || carrai.equals("OD")) {
            return "SL_WEB";
        }

        return null;
    }

}
